package com.yfd.platform.modules.experimentalData.service.impl;

import cn.hutool.core.collection.CollUtil;


import java.nio.charset.StandardCharsets;
import java.nio.file.*;
import java.nio.file.attribute.BasicFileAttributes;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


import cn.hutool.json.JSONUtil;
import com.amazonaws.services.s3.model.S3Object;
import com.amazonaws.services.s3.model.S3ObjectInputStream;
import com.amazonaws.util.IOUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yfd.platform.component.ServerSendEventServer;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.experimentalData.domain.*;
import com.yfd.platform.modules.experimentalData.enums.CompressionFormat;
import com.yfd.platform.modules.experimentalData.mapper.TsFilesMapper;
import com.yfd.platform.modules.experimentalData.service.ITsFilesService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.storage.context.StorageSourceContext;
import com.yfd.platform.modules.storage.mapper.StorageSourceConfigMapper;
import com.yfd.platform.modules.storage.model.entity.StorageSourceConfig;
import com.yfd.platform.modules.storage.model.enums.FileTypeEnum;
import com.yfd.platform.modules.storage.model.request.BatchDeleteRequest;
import com.yfd.platform.modules.storage.model.request.NewFolderRequest;
import com.yfd.platform.modules.storage.model.request.RenameFileRequest;
import com.yfd.platform.modules.storage.model.result.FileItemResult;
import com.yfd.platform.modules.storage.service.base.AbstractBaseFileService;
import com.yfd.platform.system.domain.LoginUser;
import com.yfd.platform.system.domain.SysDictionaryItems;
import com.yfd.platform.system.mapper.SysDictionaryItemsMapper;
import com.yfd.platform.utils.StringUtils;
import io.netty.channel.ChannelInboundHandlerAdapter;
import org.apache.commons.compress.archivers.sevenz.SevenZArchiveEntry;
import org.apache.commons.compress.archivers.sevenz.SevenZFile;
import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.archivers.tar.TarArchiveOutputStream;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipArchiveOutputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorInputStream;
import org.apache.commons.compress.compressors.gzip.GzipCompressorOutputStream;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.xml.crypto.Data;
import java.io.*;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.zip.*;

import org.apache.commons.codec.binary.Hex;


/**
 * <p>
 * 试验任务文档表 服务实现类
 * </p>
 *
 * @author LiMengNan
 * @since 2025-01-21
 */
@Service
public class TsFilesServiceImpl extends ServiceImpl<TsFilesMapper, TsFiles> implements ITsFilesService {
    private static final Logger LOGGER = LoggerFactory.getLogger(ChannelInboundHandlerAdapter.class);

    private int time = 1;

    //试验任务文档表 Mapper
    @Resource
    private TsFilesMapper tsFilesMapper;

    @Resource
    private StorageSourceContext storageSourceContext;

    @Resource
    private StorageSourceConfigMapper storageSourceConfigMapper;

    @Resource
    private SysDictionaryItemsMapper sysDictionaryItemsMapper;
    // 全局变量，用于存储当前任务的 Future
    private Future<?> currentTaskFuture;

    private final ExecutorService executorService = Executors.newSingleThreadExecutor();

    // 从数据库获取的压缩类型列表
    private List<String> compressSuffixes;
    private final Set<String> addedEntries = new HashSet<>();

    /**********************************
     * 用途说明: 分页查询试验数据管理-文档内容
     * 参数说明
     *id            id
     * fileName      文件名称
     * startDate     (开始日期)
     * endDate      （结束日期）
     * keywords      关键字
     *nodeId         节点ID
     *taskId          任务ID
     *isFile          文件夹、文件区分
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public Page<TsFiles> getTsFilesPage(String id, String fileName, String startDate, String endDate, String keywords, String nodeId, String taskId, String fileName1, String childNode, Page<TsFiles> page) throws Exception {
        //查询字典表获取压缩文件后缀
        QueryWrapper<SysDictionaryItems> queryWrapperSysDictionary = new QueryWrapper<>();
        queryWrapperSysDictionary.eq("parentcode", "compressType");
        queryWrapperSysDictionary.orderByAsc("orderno");
        List<SysDictionaryItems> sysDictionaryItems = sysDictionaryItemsMapper.selectList(queryWrapperSysDictionary);


        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        // 固定条件过滤
        if (StringUtils.isNotBlank(taskId)) queryWrapper.eq("task_id", taskId);
        if (StringUtils.isNotBlank(nodeId)) queryWrapper.eq("node_id", nodeId);

        // 模糊搜索
        if (StringUtils.isNotBlank(fileName)) queryWrapper.like("file_name", fileName);
        if (StringUtils.isNotBlank(keywords)) queryWrapper.like("keywords", keywords);

        // 时间范围过滤
        if (StringUtils.isNotBlank(startDate)) queryWrapper.ge("upload_time", startDate);
        if (StringUtils.isNotBlank(endDate)) queryWrapper.le("upload_time", endDate);
        boolean hasSearchCondition = StringUtils.isNotBlank(fileName) || StringUtils.isNotBlank(keywords) || StringUtils.isNotBlank(startDate) && StringUtils.isNotBlank(endDate);

        // 处理层级关系条件
        if (StringUtils.isNotBlank(id)) {
            // 有ID时：根据childNode决定是否递归子节点   如果filename
            if (childNode != null && childNode.equals("0") && hasSearchCondition) {
                List<String> childIds = getAllChildIds(id);
                queryWrapper.in("parent_id", childIds);
            } else {
                // 其他情况：childNode=1 或 无搜索条件时 按当前层级查询
                queryWrapper.eq("parent_id", id);
            }
        } else {
            //如果说ID是空 说明是顶级节点
            if (childNode != null) {
                if (childNode.equals("0")) {
                    if (hasSearchCondition) {
                        // 递归获取根目录下所有子节点ID（包含多级）
                        List<String> allChildIds = getAllChildIds("00"); // 注意这里传入字符串"00"
                        allChildIds.add("00"); // 包含根目录的直接子节点
                        queryWrapper.in("parent_id", allChildIds);
                    } else {
                        // 无搜索条件时，仅查询根目录的直接子节点
                        queryWrapper.eq("parent_id", "00");
                    }
                } else if (childNode.equals("1")) {
                    // 始终只查根目录的直接子节点
                    queryWrapper.eq("parent_id", "00");
                }
            }

//            // 无ID时：childNode=1则查根目录，childNode=0查全表
//            if (childNode != null && childNode.equals("1")) {
//                queryWrapper.eq("parent_id", "00");
//            }
        }
        //
        queryWrapper.isNotNull("work_path");
        queryWrapper.ne("work_path", "");
        //排序
        queryWrapper.orderByDesc("is_file");//时间
//        queryWrapper.orderByDesc("upload_time");//时间
        //分页查询
        Page<TsFiles> tsFilesPage = tsFilesMapper.selectPage(page, queryWrapper);
        if (tsFilesPage == null) {
            tsFilesPage = new Page<>();
            tsFilesPage.setRecords(new ArrayList<>()); // 确保 records 被初始化
        }
        List<TsFiles> records = tsFilesPage.getRecords();
        for (TsFiles tsFiles : records) {

            // 获取原始路径和 nodeId
            String workPath = tsFiles.getWorkPath();

            // 拼接需要去掉的部分
            String toRemove = File.separator + nodeId;


            String fileNameData = tsFiles.getFileName();
            //主要是用于文件路径加名称
            String path = workPath + fileNameData;
            //准备获取文件的信息
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
            FileItemResult fileItemResult = fileService.getFileItem(path);
            if (fileItemResult == null || fileItemResult.getName() == null) {
                LOGGER.error("{}文件没有上传到工作空间，请重新选择上传", fileNameData);
            }
            tsFiles.setUrl(fileItemResult.getUrl());
            //如果是压缩文件 类型就给zip
            boolean isValid = hasValidExtension(fileItemResult.getName(), sysDictionaryItems);
            if (isValid) {
                tsFiles.setType("ZIP");
            } else {
                tsFiles.setType(fileItemResult.getType().getValue());
            }
            if (tsFiles.getUpdateTime() == null) {
                tsFiles.setUpdateTime(tsFiles.getUploadTime());
            }

            // 将路径分隔符统一为 "/"
            String standardizedWorkPath = workPath.replace("\\", "/");
            String standardizedToRemove = toRemove.replace("\\", "/");

            // 检查 standardizedWorkPath 是否包含 standardizedToRemove
            if (standardizedWorkPath.contains(standardizedToRemove)) {
                // 去掉目标部分
                String newWorkPath = standardizedWorkPath.replace(standardizedToRemove, "");
                // 确保路径最后保留一个 '/'
                if (!newWorkPath.endsWith("/")) {
                    newWorkPath += "/";
                }
                // 更新路径
                tsFiles.setWorkPath(newWorkPath);  // 更新路径
            }
        }
        tsFilesPage.setRecords(records); // 同步到 tsFilesPage
        System.out.println("Updated records: " + records);
        return tsFilesPage;

    }

    public boolean hasValidExtension(String name, List<SysDictionaryItems> sysDictionaryItems) {
        // 如果传入的文件名为空，返回 false
        if (name == null) {
            return false;
        }
        // 如果传入的字典表集合为空，返回 false
        if (sysDictionaryItems == null || sysDictionaryItems.isEmpty()) {
            return false;
        }
        // 获取文件的扩展名
        String fileExtension = getFileExtension(name);
        for (SysDictionaryItems sysDictionaryItems1 : sysDictionaryItems) {
            String dictName = "." + sysDictionaryItems1.getDictName();
            if (fileExtension.equalsIgnoreCase(dictName)) {
                // 如果文件扩展名与字典表中的扩展名匹配，返回 true
                return true;
            }
        }
        // 如果没有匹配的扩展名，返回 false
        return false;
    }

    // 获取文件扩展名的方法
    private String getFileExtension(String filename) {
        if (filename != null && filename.contains(".")) {
            return filename.substring(filename.lastIndexOf("."));
        }
        return ""; // 如果没有扩展名，返回空字符串
    }

    // 递归获取所有子节点ID（包含隔代子节点）
    private List<String> getAllChildIds(String parentId) {
        List<String> childIds = new ArrayList<>();
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().select("id").eq("parent_id", parentId).eq("is_file", "FOLDER") // 只遍历文件夹节点
        );
        for (TsFiles child : children) {
            childIds.add(child.getId());
            childIds.addAll(getAllChildIds(child.getId()));
        }
        return childIds;
    }

    /**********************************
     * 用途说明:   查询实验数据管理文件夹
     * 参数说明
     * pageNum 当前页
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回分页查询结果
     ***********************************/
    @Override
    public List<TsFiles> getsListTsFiles(String id, String path, String nodeId, String taskId) {
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_file", "FOLDER");
        if (StringUtils.isNotBlank(id)) {
            queryWrapper.eq("parent_id", id);
        }
        if (StringUtils.isNotBlank(path)) {
            queryWrapper.eq("work_path", path);
        }
        if (StringUtils.isNotBlank(nodeId)) {
            queryWrapper.eq("node_id", nodeId);
        }
        if (StringUtils.isNotBlank(taskId)) {
            queryWrapper.eq("task_id", taskId);
        }
        queryWrapper.isNotNull("work_path");
        List<TsFiles> tsFiles = tsFilesMapper.selectList(queryWrapper);

        for (TsFiles tsFiles1 : tsFiles) {
            // 获取原始路径和 nodeId
            String workPath = tsFiles1.getWorkPath();
            // 拼接需要去掉的部分
            String toRemove = File.separator + nodeId;
            // 将路径分隔符统一为 "/"
            String standardizedWorkPath = workPath.replace("\\", "/");
            String standardizedToRemove = toRemove.replace("\\", "/");

            // 检查 standardizedWorkPath 是否包含 standardizedToRemove
            if (standardizedWorkPath.contains(standardizedToRemove)) {
                // 去掉目标部分
                String newWorkPath = standardizedWorkPath.replace(standardizedToRemove, "");
                // 确保路径最后保留一个 '/'
                if (!newWorkPath.endsWith("/")) {
                    newWorkPath += "/";
                }
                // 更新路径
                tsFiles1.setWorkPath(newWorkPath);  // 更新路径
            }
        }
        return tsFiles;
    }


    /***********************************
     * 用途说明：新增试验数据管理-文档内容
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    public ResponseResult addTsFiles(TsFiles tsFiles) {

        Boolean value = true;
        //文件名称和大小   因为支持多个上传所以用，分隔
        List<String> names = Arrays.asList(tsFiles.getFileName().split(","));
        List<String> sizes = Arrays.asList(tsFiles.getFileSize().split(","));
        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        // 数据校验
        if (names.size() != sizes.size()) {
            LOGGER.error("文件名称和文件大小的列表长度不一致");
            return ResponseResult.error("文件名称和文件大小的列表长度不一致！");
        }

//
//        //判断文件夹是否创建
//        AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
//        boolean flag = fileService.isFolderCreated(File.separator + tsFiles.getNodeId());
        //如果是false 说明没有创建  那就新建一个文件夹
//        if(!flag){
//            //本地创建文件夹
//            NewFolderRequest newFolderRequest = new NewFolderRequest();
//            newFolderRequest.setStorageKey("local");
//            newFolderRequest.setPath(tsFiles.getWorkPath());
//            newFolderRequest.setPassword(null);
//            newFolderRequest.setName(tsFiles.getFileName());
//
//            AbstractBaseFileService<?> fileServiceData = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
//            boolean flagData = fileServiceData.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
//            if(!flagData){
//                LOGGER.error("创建节点文件夹失败！");
//                return ResponseResult.error("新增文件失败!");
//            }
//        }

        List<TsFiles> filesToSave = new ArrayList<>();
        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);
        tsFiles.setUploadTime(currentTime);

        for (int i = 0; i < names.size(); i++) {
            String name = names.get(i).trim();
            String sizeStr = sizes.get(i).trim();

            String pathAndName = tsFiles.getWorkPath() + name;
            //准备获取文件的信息
            AbstractBaseFileService<?> fileServicefile = storageSourceContext.getByStorageKey("local");
            FileItemResult fileItemResult = fileServicefile.getFileItem(pathAndName);
            if (fileItemResult == null || fileItemResult.getName() == null) {
                return ResponseResult.error(name + "文件没有上传到工作空间，请重新选择上传！");
            }

            //通过节点ID  任务ID   路径  上级ID 文件名称 查询是否重复
            QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("node_id", tsFiles.getNodeId());
            queryWrapper.eq("task_id", tsFiles.getTaskId());
            queryWrapper.eq("work_path", tsFiles.getWorkPath());
            queryWrapper.eq("parent_id", tsFiles.getParentId());
            queryWrapper.eq("file_name", name);
            int count = tsFilesMapper.selectCount(queryWrapper);
            // 大于0说明 区域名称重复
            if (count > 0) {
                return ResponseResult.error("文件已存在！");
            }
            // 校验文件大小是否可以转换成数值
            try {
                TsFiles files1 = new TsFiles();
                files1.setTaskId(tsFiles.getTaskId());
                files1.setIsFile(tsFiles.getIsFile());
                files1.setParentId(tsFiles.getParentId());
                files1.setNodeId(tsFiles.getNodeId());
                files1.setWorkPath(tsFiles.getWorkPath());
                files1.setBackupPath(tsFiles.getBackupPath());
                files1.setKeywords(tsFiles.getKeywords());
                files1.setDescription(tsFiles.getDescription());
                files1.setUploadTime(tsFiles.getUploadTime());
                files1.setUploader(loginuser.getUsername());
                files1.setFileName(name);
                files1.setFileSize(sizeStr);
                filesToSave.add(files1);
            } catch (NumberFormatException e) {
                LOGGER.error("文件大小必须是有效的数字");
            }
        }
        if (filesToSave.size() > 0) {
            //循环新增
            for (TsFiles filess : filesToSave) {
                int valueAdded = tsFilesMapper.insert(filess);
                if (valueAdded == 1) {
                    LOGGER.info("表结构和本地都创建文件夹成功");
                    value = true;
                } else {
                    LOGGER.error("本地创建文件夹成功，表结构新增失败");
                    value = false;
                }
            }
        }
        if (value) {
            return ResponseResult.success("新增文件成功！");
        } else {
            return ResponseResult.error("新增文件失败！");
        }

    }


    /***********************************
     * 用途说明：新增试验数据管理-文件夹
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明:  com.yfd.platform.config.ResponseResult 返回新增成功或者失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public ResponseResult addTsFile(TsFiles tsFiles) throws IOException {

        if (tsFiles.getIsFile().equals("FILE")) {
            StorageSourceConfig config = storageSourceConfigMapper.selectOne(new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
            String basePath = config.getValue() + tsFiles.getWorkPath();
            // 拼接完整文件路径

            // 拼接完整的文件路径
            Path filePath = Paths.get(basePath, tsFiles.getFileName() + ".txt");

            // 确保目录存在，如果不存在则创建目录
            Files.createDirectories(filePath.getParent());

            // 使用 Files.write() 写入字符串内容
            Files.write(filePath, "默认内容".getBytes(StandardCharsets.UTF_8));
            tsFiles.setFileName(tsFiles.getFileName() + ".txt");
            UsernamePasswordAuthenticationToken authentication =
                    (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
            LoginUser loginuser = (LoginUser) authentication.getPrincipal();
            // 设置当前时间
            LocalDateTime now = LocalDateTime.now();
            // 转换为 Timestamp
            Timestamp currentTime = Timestamp.valueOf(now);
            tsFiles.setUploadTime(currentTime);
            tsFiles.setUploader(loginuser.getUsername());
            tsFiles.setFileSize("0");
            int valueAdded = tsFilesMapper.insert(tsFiles);
            if (valueAdded == 1) {
                LOGGER.info("文件创建成功");
                return ResponseResult.success();
            } else {
                LOGGER.error("文件创建失败");
                return ResponseResult.error();
            }
        } else {
            // 校验文件名是否包含非法字符
            String fileName = tsFiles.getFileName();
            if (containsInvalidCharacters(fileName)) {
                return ResponseResult.error("文件名包含非法字符（<>:\"/\\|?*）！");
            }

            //上传人是当前登录人
            UsernamePasswordAuthenticationToken authentication =
                    (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
            LoginUser loginuser = (LoginUser) authentication.getPrincipal();
            // 设置当前时间
            LocalDateTime now = LocalDateTime.now();
            // 转换为 Timestamp
            Timestamp currentTime = Timestamp.valueOf(now);
            tsFiles.setUploadTime(currentTime);
            tsFiles.setUploader(loginuser.getUsername());
            tsFiles.setFileSize("0");
            QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("node_id", tsFiles.getNodeId());
            queryWrapper.eq("task_id", tsFiles.getTaskId());
            queryWrapper.eq("parent_id", tsFiles.getParentId());
            queryWrapper.eq("file_name", tsFiles.getFileName());
            int count = tsFilesMapper.selectCount(queryWrapper);
            // 大于0说明 区域名称重复
            if (count > 0) {
                return ResponseResult.error("文件夹名称已存在！");
            }

//        //判断文件夹是否创建
//        AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
//        boolean flag = fileService.isFolderCreated(File.separator + tsFiles.getNodeId());
//        //如果是false 说明没有创建  那就新建一个文件夹
//        if(!flag){
//            //本地创建文件夹
//            NewFolderRequest newFolderRequest = new NewFolderRequest();
//            newFolderRequest.setStorageKey("local");
//            newFolderRequest.setPath(tsFiles.getWorkPath());
//            newFolderRequest.setPassword(null);
//            newFolderRequest.setName(tsFiles.getFileName());
//
//            AbstractBaseFileService<?> fileServiceData = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
//            boolean flagData = fileServiceData.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
//            if(!flagData){
//                LOGGER.error("创建节点文件夹失败！");
//                return ResponseResult.error("新增文件夹失败!");
//            }
//        }


            //本地创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setStorageKey("local");
            newFolderRequest.setPath(tsFiles.getWorkPath());
            newFolderRequest.setPassword(null);
            newFolderRequest.setName(tsFiles.getFileName());

            AbstractBaseFileService<?> fileServicefolder = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flagfolder = fileServicefolder.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flagfolder) {
//            tsFiles.setWorkPath(File.separator + tsFiles.getNodeId() + tsFiles.getWorkPath());
                int valueAdded = tsFilesMapper.insert(tsFiles);
                if (valueAdded == 1) {
                    LOGGER.info("表结构和本地都创建文件夹成功");
                    return ResponseResult.success();
                } else {
                    LOGGER.error("本地创建文件夹成功，表结构新增失败");
                    return ResponseResult.error();
                }
            } else {
                LOGGER.error("本地创建文件夹失败");
                return ResponseResult.error();
            }
        }
    }

    // 校验文件名是否包含非法字符
    private boolean containsInvalidCharacters(String fileName) {
        // 定义非法字符的正则表达式
        String regex = "[<>:\"/\\\\|?*]";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(fileName);
        return matcher.find();
    }

    /**********************************
     * 用途说明: 修改试验数据管理-文档内容
     * 参数说明
     * TsFiles     文档内容
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public ResponseResult updateTsFiles(TsFiles tsFiles) {


        // 校验文件名是否包含非法字符
        String fileName = tsFiles.getFileName();
        if (containsInvalidCharacters(fileName)) {
            return ResponseResult.error("文件名包含非法字符（<>:\"/\\|?*）！");
        }

        // 修改之前查询表中的文件名是否修改，如果发生变动先修改 minio 然后再修改表结构
        TsFiles filesData = tsFilesMapper.selectById(tsFiles.getId());

        // 判断文件名是否修改  不一致需要修改
        if (!tsFiles.getFileName().equals(filesData.getFileName())) {

            //校验文件夹会不会重命名
            if ("FOLDER".equals(tsFiles.getIsFile())) {
                //如果是文件夹 递归修改它下面的所有文件的路径
                QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", tsFiles.getNodeId());
                queryWrapper.eq("task_id", tsFiles.getTaskId());
                queryWrapper.eq("work_path", tsFiles.getWorkPath());
                queryWrapper.eq("parent_id", tsFiles.getParentId());
                queryWrapper.eq("file_name", tsFiles.getFileName());
                int count = tsFilesMapper.selectCount(queryWrapper);
                // 大于0说明 区域名称重复
                if (count > 0) {
                    return ResponseResult.error("文件夹名称已存在！");
                }

                //查询数据源
                RenameFileRequest renameFileRequest = new RenameFileRequest();
                renameFileRequest.setName(filesData.getFileName());
                renameFileRequest.setNewName(tsFiles.getFileName());
                renameFileRequest.setPassword("");
                renameFileRequest.setPath(filesData.getWorkPath());
                renameFileRequest.setStorageKey("local");
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(renameFileRequest.getStorageKey());
                Boolean flag = fileService.renameFile(renameFileRequest.getPath(), renameFileRequest.getName(), renameFileRequest.getNewName());
                //如果是true  说明至少有一个修改了文件夹
                if (flag) {
                    //递归下面所有的文件文件夹  路径都需要修改
                    List<TsFiles> resultList = new ArrayList<>();

                    if (tsFiles.getBackupPath() != null && tsFiles.getBackupPath() != "") {
                        // 设置当前时间
                        LocalDateTime now = LocalDateTime.now();
                        // 转换为 Timestamp
                        Timestamp currentTime = Timestamp.valueOf(now);
                        //新通过id找到表中现有的 修改也是修改表中现有的数据   新增的一条就是重命名以后的  所有备份空间路径为空    原来的哪条数据把工作空间路径改成空
                        TsFiles tsFiles2 = tsFilesMapper.selectById(tsFiles.getId());//查询原来的数据   把原来的数据
                        tsFiles2.setWorkPath("");

                        TsFiles tsFiles1 = new TsFiles();
                        tsFiles1.setNodeId(tsFiles.getNodeId());
                        tsFiles1.setTaskId(tsFiles.getTaskId());
                        tsFiles1.setIsFile(tsFiles.getIsFile());
                        tsFiles1.setParentId(tsFiles.getParentId());
                        tsFiles1.setFileName(tsFiles.getFileName());
                        tsFiles1.setFileSize(tsFiles.getFileSize());
                        tsFiles1.setWorkPath(tsFiles.getWorkPath());
                        tsFiles1.setBackupPath("");
                        tsFiles1.setKeywords(tsFiles.getKeywords());
                        tsFiles1.setDescription(tsFiles.getDescription());
                        tsFiles1.setUploadTime(currentTime);
                        tsFiles1.setUploader(tsFiles.getUploader());
                        tsFiles1.setUpdateTime(currentTime);
                        int valueInsert = tsFilesMapper.insert(tsFiles1);
                        if (valueInsert == 1) {
                            int valueUpdate = tsFilesMapper.updateById(tsFiles2);
                            // 获取当前文件夹的完整路径（如 "/147/222/"）
                            String oldFullPath = filesData.getWorkPath() + filesData.getFileName() + "/";
                            String newFullPath = filesData.getWorkPath() + tsFiles.getFileName() + "/";
                            querySubFilesAndUpdatePaths(resultList, tsFiles2.getId(), oldFullPath, newFullPath, tsFiles1.getId());
                            if (valueUpdate == 1) {
                                LOGGER.info("local和minio修改成功，表结构修改成功");
                                return ResponseResult.success();
                            } else {
                                LOGGER.error("local和minio修改成功，表结构修改失败");
                                return ResponseResult.error();
                            }
                        } else {
                            LOGGER.error("local和minio修改成功，表结构修改失败");
                            return ResponseResult.error();
                        }


                    } else {
                        // 如果备份空间为空  直接修改数据库
                        int valueUpdate = tsFilesMapper.updateById(tsFiles);
                        if (valueUpdate == 1) {
                            LOGGER.info("local和minio修改成功，表结构修改成功");
                            return ResponseResult.success();
                        } else {
                            LOGGER.error("local和minio修改成功，表结构修改失败");
                            return ResponseResult.error();
                        }
                    }
                } else {
                    LOGGER.error("local和minio修改失败");
                    return ResponseResult.error();
                }
            } else {
                //如果是文件   直接修改就可以
                QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
                queryWrapper.eq("node_id", tsFiles.getNodeId());
                queryWrapper.eq("task_id", tsFiles.getTaskId());
                queryWrapper.eq("work_path", tsFiles.getWorkPath());
                queryWrapper.eq("parent_id", tsFiles.getParentId());
                queryWrapper.eq("file_name", tsFiles.getFileName());
                int count = tsFilesMapper.selectCount(queryWrapper);
                // 大于0说明 区域名称重复
                if (count > 0) {
                    return ResponseResult.error("文件名称已存在！");
                }
                //查询数据源
                RenameFileRequest renameFileRequest = new RenameFileRequest();
                renameFileRequest.setName(filesData.getFileName());
                renameFileRequest.setNewName(tsFiles.getFileName());
                renameFileRequest.setPassword("");
                renameFileRequest.setPath(filesData.getWorkPath());
                renameFileRequest.setStorageKey("local");
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(renameFileRequest.getStorageKey());
                Boolean flag = fileService.renameFile(renameFileRequest.getPath(), renameFileRequest.getName(), renameFileRequest.getNewName());
                //如果是true  说明至少有一个修改了文件
                if (flag) {
                    //如果备份路径不为空  那么就先增加一条  然后修改本地的
                    if (tsFiles.getBackupPath() != null && tsFiles.getBackupPath() != "") {
                        // 设置当前时间
                        LocalDateTime now = LocalDateTime.now();
                        // 转换为 Timestamp
                        Timestamp currentTime = Timestamp.valueOf(now);
                        //新通过id找到表中现有的 修改也是修改表中现有的数据   新增的一条就是重命名以后的  所有备份空间路径为空    原来的哪条数据把工作空间路径改成空
                        TsFiles tsFiles2 = tsFilesMapper.selectById(tsFiles.getId());//查询原来的数据   把原来的数据
                        tsFiles2.setWorkPath("");

                        TsFiles tsFiles1 = new TsFiles();
                        tsFiles1.setNodeId(tsFiles.getNodeId());
                        tsFiles1.setTaskId(tsFiles.getTaskId());
                        tsFiles1.setIsFile(tsFiles.getIsFile());
                        tsFiles1.setParentId(tsFiles.getParentId());
                        tsFiles1.setFileName(tsFiles.getFileName());
                        tsFiles1.setFileSize(tsFiles.getFileSize());
                        tsFiles1.setWorkPath(tsFiles.getWorkPath());
                        tsFiles1.setBackupPath("");
                        tsFiles1.setKeywords(tsFiles.getKeywords());
                        tsFiles1.setDescription(tsFiles.getDescription());
                        tsFiles1.setUploadTime(currentTime);
                        tsFiles1.setUploader(tsFiles.getUploader());
                        tsFiles1.setUpdateTime(currentTime);
                        int valueInsert = tsFilesMapper.insert(tsFiles1);
                        if (valueInsert == 1) {
                            int valueUpdate = tsFilesMapper.updateById(tsFiles2);
                            if (valueUpdate == 1) {
                                LOGGER.info("local和minio修改成功，表结构修改成功");
                                return ResponseResult.success();
                            } else {
                                LOGGER.error("local和minio修改成功，表结构修改失败");
                                return ResponseResult.error();
                            }
                        } else {
                            LOGGER.error("local和minio修改成功，表结构修改失败");
                            return ResponseResult.error();
                        }


                    } else {
                        // 如果备份空间为空  直接修改数据库
                        int valueUpdate = tsFilesMapper.updateById(tsFiles);
                        if (valueUpdate == 1) {
                            LOGGER.info("local和minio修改成功，表结构修改成功");
                            return ResponseResult.success();
                        } else {
                            LOGGER.error("local和minio修改成功，表结构修改失败");
                            return ResponseResult.error();
                        }
                    }


                } else {
                    LOGGER.error("local和minio修改失败");
                    return ResponseResult.error();
                }
            }
        } else {
            // 如果文件名没有修改，仅更新数据库
            int valueUpdate = tsFilesMapper.updateById(tsFiles);
            if (valueUpdate == 1) {
                return ResponseResult.success();
            } else {
                return ResponseResult.error();
            }
        }
    }

    //
    private void querySubFilesAndUpdatePaths(List<TsFiles> resultList, String id, String oldName, String newName, String newid) {
        // 构造 QueryWrapper 查询当前文件夹下的文件
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", id);
        // 查询当前文件夹下的所有子项
        List<TsFiles> subFiles = tsFilesMapper.selectList(queryWrapper);
        // 确保 subFiles 不为空
        if (subFiles == null || subFiles.isEmpty()) {
            return;
        }

        // 将查询到的结果添加到结果列表
        resultList.addAll(subFiles);

        // 遍历每个子项，更新路径并递归查询其子项
        for (TsFiles subFile : subFiles) {
            String oldWorkPath = subFile.getWorkPath();
            // 使用正则表达式替换目标文件夹名称
            String newWorkPath = oldWorkPath.replaceAll(oldName, newName);

            //判断备份空间路径是否为空   如果不为
            if (subFile.getBackupPath() != null && subFile.getBackupPath() != "") {
                // 设置当前时间
                LocalDateTime now = LocalDateTime.now();
                // 转换为 Timestamp
                Timestamp currentTime = Timestamp.valueOf(now);
                //新通过id找到表中现有的 修改也是修改表中现有的数据   新增的一条就是重命名以后的  所有备份空间路径为空    原来的哪条数据把工作空间路径改成空
                TsFiles tsFiles2 = tsFilesMapper.selectById(subFile.getId());//查询原来的数据   把原来的数据
                tsFiles2.setWorkPath("");

                TsFiles tsFiles1 = new TsFiles();
                tsFiles1.setNodeId(subFile.getNodeId());
                tsFiles1.setTaskId(subFile.getTaskId());
                tsFiles1.setIsFile(subFile.getIsFile());
                tsFiles1.setParentId(newid);
                tsFiles1.setFileName(subFile.getFileName());
                tsFiles1.setFileSize(subFile.getFileSize());
                tsFiles1.setWorkPath(newWorkPath);
                tsFiles1.setBackupPath("");
                tsFiles1.setKeywords(subFile.getKeywords());
                tsFiles1.setDescription(subFile.getDescription());
                tsFiles1.setUploadTime(currentTime);
                tsFiles1.setUploader(subFile.getUploader());
                tsFiles1.setUpdateTime(currentTime);
                int valueInsert = tsFilesMapper.insert(tsFiles1);
                if (valueInsert == 1) {
                    int valueUpdate = tsFilesMapper.updateById(tsFiles2);
                    if (valueUpdate == 1) {

                    }
                }

            } else {
                // 更新数据库中的路径
                subFile.setWorkPath(newWorkPath);
                subFile.setParentId(newid);
                tsFilesMapper.updateById(subFile);
                // 递归查询每个子项的子文件
                if ("FOLDER".equals(subFile.getIsFile())) {
                    querySubFilesAndUpdatePaths(resultList, subFile.getId(), oldName, newName, subFile.getId());
                }
            }
        }
    }

    /**********************************
     * 用途说明: 批量删除试验数据管理-文档内容
     * 参数说明  id 文档内容ID
     * 参数说明  type local还是minio
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回批量删除成功或失败
     ***********************************/
    @Override
    @Transactional(rollbackFor = Exception.class)// 添加事务注解，遇到异常时回滚
    public String deleteTsFilesByIds(List<String> dataset, String type) {
        List<TsFiles> filesList = tsFilesMapper.selectBatchIds(dataset);

        int LocalSuccessCount = 0, LocalFailCount = 0, Localtotal = CollUtil.size(dataset);
        //Todo  最直接的办法   循环出来 一条一条删除
        for (TsFiles files : filesList) {

            //判断是文件还是文件夹
            if ("FOLDER".equals(files.getIsFile())) {
                //如果是文件夹
                List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
                BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
                deleteItemData.setName(files.getFileName());
                deleteItemData.setPassword("");
                if ("local".equals(type)) {
                    deleteItemData.setPath(files.getWorkPath());
                } else {
                    deleteItemData.setPath(files.getBackupPath());
                }
                deleteItemData.setType(FileTypeEnum.FOLDER);
                deleteItemList.add(deleteItemData);
                //首先通过ID集合查询所有的内容  然后放到List<DeleteItem> deleteItems里面  放好以后删除数据库  然后删除minio
                BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
                batchDeleteRequest.setDeleteItems(deleteItemList);
                batchDeleteRequest.setStorageKey(type);

                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
                List<BatchDeleteRequest.DeleteItem> deleteItems = batchDeleteRequest.getDeleteItems();

                int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItems);
                for (BatchDeleteRequest.DeleteItem deleteItem : deleteItems) {

                    boolean flag = false;
                    try {
                        if (deleteItem.getType() == FileTypeEnum.FILE) {
                            flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                        } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                            flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                        }

                        if (flag) {
                            deleteSuccessCount++;
                        } else {
                            deleteFailCount++;
                        }
                    } catch (Exception e) {
                        LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                        deleteFailCount++;
                    }
                }

                if (totalCount > 1) {
                    //return ResponseResult.success("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                } else {
                    //return totalCount == deleteSuccessCount ? ResponseResult.success("删除成功") : ResponseResult.error("删除失败");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                }

                //如果是1 说明成功删除
                if (deleteSuccessCount >= 1) {
                    int valueDelete = deleteFolder(files.getId(), type);
                    if (valueDelete > 0) {
                        LocalSuccessCount = valueDelete;
                        LOGGER.info("表结构成功删除");
                    } else {
                        System.out.println("没有记录被删除");
                    }
                } else {
                    LocalFailCount++;
                }
            } else {
                //如果是文件

                List<BatchDeleteRequest.DeleteItem> deleteItemList = new ArrayList<>();
                BatchDeleteRequest.DeleteItem deleteItemData = new BatchDeleteRequest.DeleteItem();
                deleteItemData.setName(files.getFileName());
                deleteItemData.setPassword("");
                if ("local".equals(type)) {
                    deleteItemData.setPath(files.getWorkPath());
                } else {
                    deleteItemData.setPath(files.getBackupPath());
                }
                deleteItemData.setType(FileTypeEnum.FILE);
                deleteItemList.add(deleteItemData);
                //首先通过ID集合查询所有的内容  然后放到List<DeleteItem> deleteItems里面  放好以后删除数据库  然后删除minio
                BatchDeleteRequest batchDeleteRequest = new BatchDeleteRequest();
                batchDeleteRequest.setDeleteItems(deleteItemList);
                batchDeleteRequest.setStorageKey(type);

                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(batchDeleteRequest.getStorageKey());
                List<BatchDeleteRequest.DeleteItem> deleteItems = batchDeleteRequest.getDeleteItems();

                int deleteSuccessCount = 0, deleteFailCount = 0, totalCount = CollUtil.size(deleteItems);
                for (BatchDeleteRequest.DeleteItem deleteItem : deleteItems) {

                    boolean flag = false;
                    try {
                        if (deleteItem.getType() == FileTypeEnum.FILE) {
                            flag = fileService.deleteFile(deleteItem.getPath(), deleteItem.getName());
                        } else if (deleteItem.getType() == FileTypeEnum.FOLDER) {
                            flag = fileService.deleteFolder(deleteItem.getPath(), deleteItem.getName());
                        }

                        if (flag) {
                            deleteSuccessCount++;
                        } else {
                            deleteFailCount++;
                        }
                    } catch (Exception e) {
                        LOGGER.error("删除文件/文件夹失败, 文件路径: {}, 文件名称: {}", deleteItem.getPath(), deleteItem.getName(), e);
                        deleteFailCount++;
                    }
                }

                if (totalCount > 1) {
                    //return ResponseResult.success("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");
                } else {
                    //return totalCount == deleteSuccessCount ? ResponseResult.success("删除成功") : ResponseResult.error("删除失败");
                    LOGGER.error("批量删除 " + totalCount + " 个, 删除成功 " + deleteSuccessCount + " 个, 失败 " + deleteFailCount + " 个.");

                }
                //如果是1 说明成功删除
                if (deleteSuccessCount == 1) {
                    int valueDelete = deleteFiles(files.getId(), type);
                    if (valueDelete > 0) {
                        LocalSuccessCount++;
                        LOGGER.info("表结构成功删除");
                    } else {
                        System.out.println("没有记录被删除");
                    }
                } else {
                    LocalFailCount++;
                }
            }
        }
        return "Local批量删除 " + Localtotal + " 个, 删除成功 " + LocalSuccessCount + " 个, 失败 " + LocalFailCount + " 个.";
    }

    // 调用删除方法并返回删除数量
    public int deleteFolder(String folderId, String type) {
        return deleteFolderAndSubFiles(folderId, type);  // 返回删除的记录数量
    }

    // 递归删除文件夹及其子文件，并返回删除的记录数量
    public int deleteFolderAndSubFiles(String folderId, String type) {
        int deletedCount = 0;  // 记录删除的数量

        // 查询该文件夹下的所有子文件
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", folderId); // 根据父ID查询所有子文件
        List<TsFiles> subFiles = tsFilesMapper.selectList(queryWrapper);

        // 递归删除子文件
        for (TsFiles subFile : subFiles) {
            // 递归调用删除子文件的逻辑
            deletedCount += deleteFolderAndSubFiles(subFile.getId(), type);  // 累加删除数量

            //判断是删除本地还是minio
            if ("local".equals(type)) {
                //删除本地的时候判断minio路径是否为空  如果为空直接删除  如果不为空把workPath修改成空
                if (StringUtils.isNotEmpty(subFile.getBackupPath())) {
                    subFile.setWorkPath("");
                    tsFilesMapper.updateById(subFile);
                } else {
                    // 删除子文件记录（从数据库中删除）
                    tsFilesMapper.deleteById(subFile.getId());
                }

            } else {
                //删除minio的时候判断本地路径是否为空  如果为空直接删除  如果不为空把BackupPath修改成空
                if (StringUtils.isNotEmpty(subFile.getWorkPath())) {
                    subFile.setBackupPath("");
                    tsFilesMapper.updateById(subFile);
                } else {
                    // 删除子文件记录（从数据库中删除）
                    tsFilesMapper.deleteById(subFile.getId());
                }
            }
            deletedCount++;  // 每删除一个文件，数量加1
        }

        //删除修改自身
        TsFiles tsFilesData = tsFilesMapper.selectById(folderId);
        //判断是删除本地还是minio
        if ("local".equals(type)) {
            //删除本地的时候判断minio路径是否为空  如果为空直接删除  如果不为空把workPath修改成空
            if (StringUtils.isNotEmpty(tsFilesData.getBackupPath())) {
                tsFilesData.setWorkPath("");
                tsFilesMapper.updateById(tsFilesData);
            } else {
                // 删除子文件记录（从数据库中删除）
                tsFilesMapper.deleteById(tsFilesData.getId());
            }

        } else {
            //删除minio的时候判断本地路径是否为空  如果为空直接删除  如果不为空把BackupPath修改成空
            if (StringUtils.isNotEmpty(tsFilesData.getWorkPath())) {
                tsFilesData.setBackupPath("");
                tsFilesMapper.updateById(tsFilesData);
            } else {
                // 删除子文件记录（从数据库中删除）
                tsFilesMapper.deleteById(tsFilesData.getId());
            }
        }
        deletedCount++;  // 删除文件夹本身，数量加1

        return deletedCount;  // 返回删除的数量
    }


    public int deleteFiles(String id, String type) {
        int deletedCount = 0;
        TsFiles tsFilesData = tsFilesMapper.selectById(id);
        if (tsFilesData != null) {
            //判断是删除本地还是minio
            if ("local".equals(type)) {
                //删除本地的时候判断minio路径是否为空  如果为空直接删除  如果不为空把workPath修改成空
                if (StringUtils.isNotEmpty(tsFilesData.getBackupPath())) {
                    tsFilesData.setWorkPath("");
                    tsFilesMapper.updateById(tsFilesData);
                    deletedCount++;
                } else {
                    // 删除子文件记录（从数据库中删除）
                    tsFilesMapper.deleteById(tsFilesData.getId());
                    deletedCount++;

                }

            } else {
                //删除minio的时候判断本地路径是否为空  如果为空直接删除  如果不为空把BackupPath修改成空
                if (StringUtils.isNotEmpty(tsFilesData.getWorkPath())) {
                    tsFilesData.setBackupPath("");
                    tsFilesMapper.updateById(tsFilesData);
                    deletedCount++;

                } else {
                    // 删除子文件记录（从数据库中删除）
                    tsFilesMapper.deleteById(tsFilesData.getId());
                    deletedCount++;

                }
            }
        }
        return deletedCount;
    }


    /**********************************
     * 用途说明: 压缩文件夹接口
     * 参数说明  ids 文件id数组
     * 参数说明  compressedFormat 压缩文件格式
     * 参数说明  compressedName 压缩文件名称
     * 参数说明  compressedPath 压缩文件路径
     * 参数说明  covered 是否覆盖 0 覆盖更新updateTime时间 1提示文件存在
     * 参数说明  parentId 父ID
     * 返回值说明: com.yfd.platform.config.ResponseResult
     ***********************************/
    @Override
    public String compressFolder(String ids, String compressedFormat, String compressedName, String compressedPath, String covered, String parentId) throws FileNotFoundException {
        //获取本地根路径
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);
        String returnResult = "";
        // 创建要压缩的文件夹列表
        List<Path> sourceDirs = new ArrayList<>(); // 修复：使用 ArrayList
        String[] splitIds = ids.split(",");
        // 数组转集合
        List<String> dataset = Arrays.asList(splitIds); // 使用 Arrays.asList 创建不可变列表
        List<TsFiles> filesList = tsFilesMapper.selectBatchIds(dataset);
        // 检查 filesList 是否为空
        if (filesList == null || filesList.isEmpty()) {
            throw new RuntimeException("未找到要压缩的文件或文件夹");
        }
        for (TsFiles tsFiles : filesList) {
            // 检查 tsFiles 的字段是否为空
            if (tsFiles.getWorkPath() == null || tsFiles.getFileName() == null) {
                throw new RuntimeException("文件或文件夹信息不完整");
            }

            // 构建完整路径
            String sourceDir = storageSourceConfig.getValue() + tsFiles.getWorkPath() + tsFiles.getFileName();
            Path path = Paths.get(sourceDir);
            // 检查路径是否存在
            if (!Files.exists(path)) {
                throw new FileNotFoundException("路径不存在: " + path);
            }
            // 添加到压缩列表
            sourceDirs.add(path); // 修复：使用可变集合
        }

        // 规范化压缩包存放路径
        // 如果 compressedPath 是根目录，使用当前工作目录
        Path zipFilePath = Paths.get(storageSourceConfig.getValue() + compressedPath + compressedName + "." + compressedFormat);

        // 检查目标路径是否可写
        if (!Files.isWritable(zipFilePath.getParent())) {
            throw new RuntimeException("目标路径不可写: " + zipFilePath.getParent());
        }

        String zipFileName = compressedName + "." + compressedFormat;

        // 使用 try-with-resources 确保 zipOut 在使用后关闭
        // 设置压缩级别，可以选择调整
//            zipOut.setLevel(ZipArchiveOutputStream.STORED); // 或者使用其他的压缩级别
        // 调用压缩方法
        Boolean value = compressToSameDirectory(sourceDirs, compressedFormat);
        if (value) {
            //表结构增加    nodeId, String TaskId
            TsFiles tsFiles = new TsFiles();
            tsFiles.setTaskId(filesList.get(0).getTaskId());
            tsFiles.setNodeId(filesList.get(0).getNodeId());
            tsFiles.setWorkPath(compressedPath);
            tsFiles.setIsFile("FILE");

            // 获取文件大小（字节）
            long fileSizeInBytes = zipFilePath.toFile().length();
            // 转换为 MB 并保留两位小数
            double fileSizeInMB = fileSizeInBytes / (1024.0 * 1024.0);
            String fileSizeFormatted = String.format("%.2f", fileSizeInMB); // 保留两位小数
            // 设置文件大小
            tsFiles.setFileSize(fileSizeFormatted);
            //tsFiles.setFileSize(String.valueOf(zipFilePath.toFile().length()));
            tsFiles.setParentId(parentId);
            tsFiles.setBackupPath("");
            tsFiles.setKeywords("");
            tsFiles.setDescription("");
            tsFiles.setFileName(zipFileName);

            //上传人是当前登录人
            UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
            LoginUser loginuser = (LoginUser) authentication.getPrincipal();
            // 设置当前时间
            LocalDateTime now = LocalDateTime.now();
            // 转换为 Timestamp
            Timestamp currentTime = Timestamp.valueOf(now);
            tsFiles.setUploadTime(currentTime);
            tsFiles.setUploader(loginuser.getUsername());
            // 通过 nodeid  taskid  名称查询  如果存在就不 新增

            QueryWrapper<TsFiles> queryWrapperTsFiles = new QueryWrapper<>();
            queryWrapperTsFiles.eq("node_id", filesList.get(0).getNodeId());
            queryWrapperTsFiles.eq("task_id", filesList.get(0).getTaskId());
            queryWrapperTsFiles.eq("work_path", compressedPath);
            queryWrapperTsFiles.eq("file_name", zipFileName);
            queryWrapperTsFiles.eq("parent_id", filesList.get(0).getParentId());
            TsFiles tsFilesdata = tsFilesMapper.selectOne(queryWrapperTsFiles);
            //covered 是否覆盖 0 覆盖更新updateTime时间 1提示文件存在
            if ("0".equals(covered)) {
                //如果不为空 更新updateTime时间
                if (tsFilesdata != null) {
                    tsFilesdata.setUpdateTime(currentTime);
                    tsFilesMapper.updateById(tsFilesdata);
                } else {
                    //如果为空就新增
                    int valueAdded = tsFilesMapper.insert(tsFiles);
                    if (valueAdded == 1) {
                        returnResult = "压缩成功";
                    } else {
                        returnResult = "压缩失败";
                    }
                }

            } else {
                // 1提示文件存在   如果不为空返回提示
                if (tsFilesdata != null) {
                    returnResult = "文件或文件夹已存在！";
                } else {
                    //如果为空就新增
                    int valueAdded = tsFilesMapper.insert(tsFiles);
                    if (valueAdded == 1) {
                        returnResult = "压缩成功";
                    } else {
                        returnResult = "压缩失败";
                    }
                }
            }
            returnResult = "压缩成功";
        } else {
            returnResult = "压缩失败";
        }
        return returnResult;
    }


    // ================== 核心方法改造 ==================


    private Boolean compressToSameDirectory(List<Path> sourcePaths, String compressedFormat) {
        try {
            switch (compressedFormat.toUpperCase()) {
                case "ZIP":
                    return compressToZip(sourcePaths);
                case "TAR.GZ":
                    return compressToTarGz(sourcePaths);
//                case "7Z":
//                    return compressTo7z(sourcePaths);
                default:
                    throw new IllegalArgumentException("不支持的压缩格式: " + compressedFormat);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return false;
        }
    }

    // ================== ZIP格式压缩实现 ==================


    /**
     * 压缩多个文件/目录到单个ZIP
     */
    public boolean compressToZip(List<Path> sourcePaths) {
        if (sourcePaths.isEmpty()) {
            LOGGER.warn("源路径列表为空，无法压缩");
            return false;
        }

        Path zipPath = generateZipPath(sourcePaths.get(0));
        addedEntries.clear(); // 清空历史记录

        try (ZipOutputStream zipOut = new ZipOutputStream(Files.newOutputStream(zipPath))) {
            zipOut.setLevel(Deflater.DEFAULT_COMPRESSION);

            for (Path sourcePath : sourcePaths) {
                if (Files.isDirectory(sourcePath)) {
                    // 使用唯一化的 baseDir（例如目录名）
                    String baseDir = sourcePath.getFileName().toString();
                    addDirectoryToZip(sourcePath, zipOut, baseDir);
                } else {
                    // 文件直接添加到根目录，或指定唯一子目录
                    addFileToZip(sourcePath, zipOut, "");
                }
            }

            zipOut.finish();
            return validateZip(zipPath);
        } catch (Exception e) {
            LOGGER.error("压缩失败", e);
            deleteQuietly(zipPath);
            return false;
        }
    }

    private void addDirectoryToZip(Path dir, ZipOutputStream zipOut, String baseDir) throws IOException {
        final Path sourceDir = dir;

        Files.walkFileTree(dir, new SimpleFileVisitor<Path>() {
            @Override
            public FileVisitResult preVisitDirectory(Path currentDir, BasicFileAttributes attrs) throws IOException {
                // 跳过源目录自身（由外部显式添加）
                if (currentDir.equals(sourceDir)) {
                    return FileVisitResult.CONTINUE;
                }

                // 生成相对路径的条目名称
                Path relativePath = sourceDir.relativize(currentDir);
                String entryName = baseDir + "/" + relativePath.toString().replace("\\", "/") + "/";

                // 确保条目唯一性
                addUniqueDirectoryEntry(entryName, zipOut);
                return FileVisitResult.CONTINUE;
            }

            @Override
            public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                Path relativePath = sourceDir.relativize(file);
                String entryName = baseDir + "/" + relativePath.toString().replace("\\", "/");
                addFileEntry(file, entryName, zipOut);
                return FileVisitResult.CONTINUE;
            }
        });

        // 显式添加根目录条目（确保至少存在一个目录条目）
        String rootEntry = baseDir + "/";
        addUniqueDirectoryEntry(rootEntry, zipOut);
    }

    private void addUniqueDirectoryEntry(String entryName, ZipOutputStream zipOut) throws IOException {
        if (!addedEntries.contains(entryName)) {
            ZipEntry entry = new ZipEntry(entryName);
            zipOut.putNextEntry(entry);
            zipOut.closeEntry();
            addedEntries.add(entryName);
        }
    }

    private void addFileToZip(Path file, ZipOutputStream zipOut, String baseDir) throws IOException {
        String entryName = baseDir + file.getFileName().toString();
        addFileEntry(file, entryName, zipOut);
    }

    private void addFileEntry(Path file, String entryName, ZipOutputStream zipOut) throws IOException {
        if (addedEntries.contains(entryName)) {
            LOGGER.warn("跳过重复条目: {}", entryName);
            return;
        }

        LOGGER.debug("添加文件条目: {} (大小: {} 字节)", entryName, Files.size(file));
        ZipEntry entry = new ZipEntry(entryName);
        entry.setSize(Files.size(file));
        entry.setTime(Files.getLastModifiedTime(file).toMillis());
        zipOut.putNextEntry(entry);

        try (InputStream input = Files.newInputStream(file)) {
            byte[] buffer = new byte[8192];
            int bytesRead;
            while ((bytesRead = input.read(buffer)) != -1) {
                zipOut.write(buffer, 0, bytesRead);
            }
        }

        zipOut.closeEntry();
        addedEntries.add(entryName);
    }

    private Path generateZipPath(Path sourcePath) {
        String baseName = sourcePath.getFileName().toString().replaceFirst("[.][^.]+$", "");
        return sourcePath.resolveSibling(baseName + ".zip");
    }

    private boolean validateZip(Path zipPath) {
        try (ZipFile zipFile = new ZipFile(zipPath.toFile())) {
            Enumeration<? extends ZipEntry> entries = zipFile.entries(); // 获取所有条目
            if (!entries.hasMoreElements()) {
                LOGGER.warn("ZIP 文件为空: {}", zipPath);
                return false;
            }

            while (entries.hasMoreElements()) {
                ZipEntry entry = entries.nextElement();
                try (InputStream is = zipFile.getInputStream(entry)) {
                    byte[] buffer = new byte[1024];
                    while (is.read(buffer) != -1) {
                    } // 读取条目内容
                }
            }

            LOGGER.info("ZIP 文件验证成功: {}", zipPath);
            return true;
        } catch (Exception e) {
            LOGGER.error("ZIP 文件验证失败: {}", zipPath, e);
            return false;
        }
    }

    private void deleteQuietly(Path path) {
        try {
            Files.deleteIfExists(path);
        } catch (IOException ignored) {
        }
    }

    /**
     * 生成压缩文件路径（去除源文件扩展名）
     */
    private Path getCompressedFilePath(Path sourcePath, String extension) {
        String fileName = sourcePath.getFileName().toString();
        String baseName = fileName.contains(".")
                ? fileName.substring(0, fileName.lastIndexOf('.'))
                : fileName;
        return sourcePath.resolveSibling(baseName + "." + extension);
    }


    // ================== TAR.GZ格式压缩实现 ==================
    private boolean compressToTarGz(List<Path> sourcePaths) throws IOException {
        boolean success = true;
        for (Path sourcePath : sourcePaths) {
            Path tarGzPath = getCompressedFilePath(sourcePath, "tar.gz");
            try (OutputStream fos = Files.newOutputStream(tarGzPath);
                 GzipCompressorOutputStream gzos = new GzipCompressorOutputStream(fos);
                 TarArchiveOutputStream tarOut = new TarArchiveOutputStream(gzos)) {

                tarOut.setLongFileMode(TarArchiveOutputStream.LONGFILE_POSIX);

                processEntries(sourcePath, (entryName, isDir) -> {
                    Path filePath = sourcePath.resolve(entryName);
                    TarArchiveEntry entry = new TarArchiveEntry(filePath.toFile(), entryName);
                    entry.setSize(isDir ? 0 : Files.size(filePath));
                    tarOut.putArchiveEntry(entry);

                    if (!isDir) {
                        try (InputStream input = Files.newInputStream(filePath)) {
                            IOUtils.copy(input, tarOut);
                        }
                    }

                    tarOut.closeArchiveEntry();
                });

                tarOut.finish();
            } catch (Exception e) {
                success = false;
                e.printStackTrace();
            }
        }
        return success;
    }

    // ================== 7Z格式压缩实现 ==================
//    private boolean compressTo7z(List<Path> sourcePaths) throws IOException {
//        boolean success = true;
//        for (Path sourcePath : sourcePaths) {
//            Path sevenZPath = getCompressedFilePath(sourcePath, "7z");
//            try (SevenZOutputFile sevenZOutput = new SevenZOutputFile(sevenZPath.toFile())) {
//
//                processEntries(sourcePath, (entryName, isDir) -> {
//                    SevenZArchiveEntry entry = sevenZOutput.createEntry();
//                    entry.setName(entryName);
//                    entry.setDirectory(isDir);
//
//                    if (!isDir) {
//                        Path filePath = sourcePath.resolve(entryName);
//                        entry.setSize(Files.size(filePath));
//                        sevenZOutput.putArchiveEntry(entry);
//
//                        try (InputStream input = Files.newInputStream(filePath)) {
//                            byte[] buffer = new byte[8192];
//                            int len;
//                            while ((len = input.read(buffer)) != -1) {
//                                sevenZOutput.write(buffer, 0, len);
//                            }
//                        }
//
//                        sevenZOutput.closeArchiveEntry();
//                    }
//                });
//
//            } catch (Exception e) {
//                success = false;
//                e.printStackTrace();
//            }
//        }
//        return success;
//    }

    // ================== 通用工具方法 ==================
    private interface EntryProcessor {
        void process(String entryName, boolean isDir) throws IOException;
    }

    private void processEntries(Path rootPath, EntryProcessor processor) throws IOException {
        Files.walkFileTree(rootPath, new SimpleFileVisitor<Path>() {
            @Override
            public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
                String entryName = rootPath.relativize(dir).toString().replace("\\", "/");
                if (!entryName.isEmpty()) {
                    processor.process(entryName + "/", true);
                }
                return FileVisitResult.CONTINUE;
            }

            @Override
            public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException {
                String entryName = rootPath.relativize(file).toString().replace("\\", "/");
                processor.process(entryName, false);
                return FileVisitResult.CONTINUE;
            }
        });
    }


    // ================== ZIP条目添加方法 ==================
    private void addZipDirectoryEntry(ZipArchiveOutputStream zipOut, String entryName) throws IOException {
        ZipArchiveEntry entry = new ZipArchiveEntry(entryName);
        zipOut.putArchiveEntry(entry);
        zipOut.closeArchiveEntry();
    }

    private void addZipFileEntry(ZipArchiveOutputStream zipOut, Path filePath, String entryName) throws IOException {
        ZipArchiveEntry entry = new ZipArchiveEntry(filePath.toFile(), entryName);
        entry.setSize(Files.size(filePath));
        entry.setTime(Files.getLastModifiedTime(filePath).toMillis());

        zipOut.putArchiveEntry(entry);
        try (InputStream input = Files.newInputStream(filePath)) {
            IOUtils.copy(input, zipOut);
        }
        zipOut.closeArchiveEntry();
    }


    /*************************************解压缩*******************************************/

    /**********************************
     * 用途说明: 解压缩接口
     * 参数说明  id  要解压的文件id
     * 参数说明  decompressionPath 解压缩路径
     * 参数说明  parentId 父ID
     * 返回值说明: com.yfd.platform.config.ResponseResult
     ***********************************/
    @Override
    public String decompressionFolder(String id, String decompressionPath, String parentId) {
        // 查询本地文件路径根目录（如 E:\yun）
        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);

        // 1. 获取压缩包记录
        TsFiles zipFileRecord = tsFilesMapper.selectById(id);
        String zipName = getFileNameWithoutExtension(zipFileRecord.getFileName()); // 示例：222
        try {
            // 2. 判断压缩包类型（单文件还是文件夹）
            Path zipFilePath = Paths.get(storageSourceConfig.getValue(), zipFileRecord.getWorkPath(), zipFileRecord.getFileName());
            // 3. 判断压缩包内容类型
            boolean hasFolder = hasFolder(zipFilePath);
//            // 4. 构建解压目标路径
            Path destRoot;
            if (hasFolder) {
                // 如果有文件夹，创建子目录（如 E:/yun/333/222/）
                destRoot = Paths.get(storageSourceConfig.getValue(), decompressionPath, zipName);
            } else {
                // 如果只有文件，直接解压到目标目录（如 E:/yun/333/）
                destRoot = Paths.get(storageSourceConfig.getValue(), decompressionPath);
            }
            Files.createDirectories(destRoot);

            // 5.执行解压（到子目录）
            File unzippedRoot = unzipToTemp(
                    Paths.get(storageSourceConfig.getValue(), zipFileRecord.getWorkPath(), zipFileRecord.getFileName()),
                    destRoot.toString()
            );

            // 6. 处理解压内容
            if (hasFolder) {
                // 如果有文件夹，创建文件夹记录并处理子内容
                TsFiles rootFolder = createFolderRecord(
                        zipName, // 文件夹名称222
                        parentId, // 父ID是333的ID
                        buildFolderPath(decompressionPath), // 修正为 /333/222/
                        zipFileRecord.getTaskId(),
                        zipFileRecord.getNodeId(),
                        zipFileRecord.getUploader()
                );
                String rootFolderId = rootFolder.getId();

                // 7. 处理子内容（路径从/333/222/开始）  关键点 所有的文件夹都是上级路径 文件是上级路径加名称
                processFolderContents(
                        unzippedRoot,
                        rootFolderId,
                        buildFolderPath(decompressionPath + "/" + zipName + "/"), // 修正为 /333/222/
                        zipFileRecord.getTaskId(),
                        zipFileRecord.getNodeId(),
                        zipFileRecord.getUploader()
                );
            } else {
                // 如果只有文件，直接创建文件记录
                File[] files = unzippedRoot.listFiles();
                if (files != null && files.length > 0) {
                    for (File file : files) {
                        if (file.isDirectory()) {
                            continue;
                        } else {
                            // 获取文件大小（字节）
                            long fileSizeInBytes = file.length();
                            // 转换为 MB 并保留两位小数
                            double fileSizeInMB = fileSizeInBytes / (1024.0 * 1024.0);
                            String fileSizeFormatted = String.format("%.2f", fileSizeInMB); // 保留两位小数

                            TsFiles fileRecord = createFileRecord(
                                    file.getName(),
                                    parentId,
                                    buildFolderPath(decompressionPath),
                                    zipFileRecord.getTaskId(),
                                    zipFileRecord.getNodeId(),
                                    zipFileRecord.getUploader(),
                                    fileSizeFormatted
                            );
                        }
                    }
                }

            }
            return "解压存储成功";
        } catch (Exception e) {
            LOGGER.error("解压失败: " + e.getMessage());
            throw new RuntimeException("操作失败: " + e.getMessage());
        }
    }


    /**
     * 创建文件记录
     */
    private TsFiles createFileRecord(String fileName, String parentId, String workPath,
                                     String taskId, String nodeId, String uploader, String fileSize) {

        QueryWrapper<TsFiles> query = new QueryWrapper<TsFiles>()
                .eq("node_id", nodeId)
                .eq("task_id", taskId)
                .eq("work_path", workPath)
                .eq("file_name", fileName)
                .eq("is_file", "FILE")
                .eq("parent_id", parentId);
        TsFiles existing = tsFilesMapper.selectOne(query);
        if (existing != null) {
            return existing;
        } else {
            TsFiles fileRecord = new TsFiles();
            fileRecord.setNodeId(nodeId);
            fileRecord.setTaskId(taskId);
            fileRecord.setParentId(parentId);
            fileRecord.setFileName(fileName);
            fileRecord.setWorkPath(workPath);
            fileRecord.setIsFile("FILE");
            fileRecord.setFileSize(fileSize);
            fileRecord.setUploadTime(new Timestamp(System.currentTimeMillis()));
            operateTsfeiles(fileRecord); // 插入或更新记录
            return fileRecord;
        }
    }

    /**
     * 创建文件夹记录
     */
    private TsFiles createFolderRecord(String folderName, String parentId, String workPath,
                                       String taskId, String nodeId, String uploader) {
        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        TsFiles folderRecord = new TsFiles();
        folderRecord.setNodeId(nodeId);
        folderRecord.setTaskId(taskId);
        folderRecord.setParentId(parentId);
        folderRecord.setFileName(folderName);
        folderRecord.setWorkPath(workPath);
        folderRecord.setIsFile("FOLDER");
        folderRecord.setFileSize("0");
        folderRecord.setUploadTime(new Timestamp(System.currentTimeMillis()));
        operateTsfeiles(folderRecord); // 插入或更新记录
        return folderRecord;
    }

    /**
     * 递归处理子内容（关键：区分根目录和子内容路径）
     */
    private void processFolderContents(File currentDir, String parentId, String parentWorkPath,
                                       String taskId, String nodeId, String uploader) {
        File[] children = currentDir.listFiles();
        if (children == null) return;

        for (File child : children) {
            String childName = child.getName();
            String childWorkPath = parentWorkPath + childName + (child.isDirectory() ? "/" : ""); // 修正为 /333/222/childName/
            if (child.isDirectory()) {
                // 处理子目录（示例：555）
                String folderWorkPath = parentWorkPath;

                TsFiles folderRecord = new TsFiles();
                folderRecord.setNodeId(nodeId);
                folderRecord.setTaskId(taskId);
                folderRecord.setParentId(parentId); // 父ID是222的ID
                folderRecord.setFileName(child.getName());
                folderRecord.setWorkPath(folderWorkPath); // /333/222/555/
                folderRecord.setIsFile("FOLDER");
                folderRecord.setFileSize("0");
                folderRecord.setUploadTime(new Timestamp(System.currentTimeMillis()));
                operateTsfeiles(folderRecord);

                // 递归处理子目录内容
                processFolderContents(
                        child,
                        folderRecord.getId(),
                        childWorkPath,
                        taskId,
                        nodeId,
                        uploader
                );
            } else {

                // 获取文件大小（字节）
                long fileSizeInBytes = child.length();
                // 转换为 MB 并保留两位小数
                double fileSizeInMB = fileSizeInBytes / (1024.0 * 1024.0);
                String fileSizeFormatted = String.format("%.2f", fileSizeInMB); // 保留两位小数
                // 处理文件（示例：1.txt）
                TsFiles fileRecord = new TsFiles();
                fileRecord.setNodeId(nodeId);
                fileRecord.setTaskId(taskId);
                fileRecord.setParentId(parentId); // 父ID是222的ID
                fileRecord.setFileName(child.getName());
                fileRecord.setWorkPath(parentWorkPath); // /333/222/
                fileRecord.setIsFile("FILE");
                fileRecord.setUploadTime(new Timestamp(System.currentTimeMillis()));
                fileRecord.setFileSize(fileSizeFormatted);
                operateTsfeiles(fileRecord);
//
//                tsFilesMapper.insert(fileRecord);
            }
        }
    }

    private void operateTsfeiles(TsFiles tsFiles) {
        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        String path = normalizePath(tsFiles.getWorkPath());
        tsFiles.setWorkPath(path);
        QueryWrapper<TsFiles> query = new QueryWrapper<TsFiles>()
                .eq("node_id", tsFiles.getNodeId())
                .eq("task_id", tsFiles.getTaskId())
                .eq("work_path", tsFiles.getWorkPath())
                .eq("file_name", tsFiles.getFileName())
                .eq("is_file", tsFiles.getIsFile());
        TsFiles existing = tsFilesMapper.selectOne(query);
        if (existing != null) {
            existing.setUpdateTime(new Timestamp(System.currentTimeMillis()));
            existing.setUploader(loginuser.getUsername());
            tsFilesMapper.updateById(existing);
        } else {
            tsFiles.setUploader(loginuser.getUsername());
            tsFilesMapper.insert(tsFiles);
        }
    }

    // 文件夹路径标准化（确保以/结尾）
    private String buildFolderPath(String path) {
        return path.endsWith("/") ? path : path + "/";
    }


    // 新增：统一解压入口方法（保持原有逻辑结构）
    private File unzipToTemp(Path zipFilePath, String baseDir) throws IOException {
        String fileName = zipFilePath.getFileName().toString();
        String ext = getFileExtension(fileName).toLowerCase();

        // 根据文件类型调用不同解压实现
        if (ext.equals(".zip")) {
            return unzipFile(zipFilePath, baseDir);
        } else if (fileName.endsWith(".tar.gz")) {
            return unTarGzFile(zipFilePath, baseDir);
        } else if (ext.equals(".7z")) {
            return un7zFile(zipFilePath, baseDir);
        } else {
            throw new IllegalArgumentException("不支持的压缩格式: " + ext);
        }
    }

    // 修改后的判断是否包含文件夹方法（支持多格式）
    private boolean hasFolder(Path zipFilePath) throws IOException {
        String fileName = zipFilePath.getFileName().toString();
        String ext = getFileExtension(fileName).toLowerCase();

        if (ext.equals(".zip")) {
            return hasFolderInZip(zipFilePath);
        } else if (fileName.endsWith(".tar.gz")) {
            return hasFolderInTarGz(zipFilePath);
        } else if (ext.equals(".7z")) {
            return hasFolderIn7z(zipFilePath);
        } else {
            throw new IllegalArgumentException("不支持的压缩格式: " + ext);
        }
    }


    // 保持原有ZIP判断方法不变
    private boolean hasFolderInZip(Path zipFilePath) throws IOException {
        try (ZipFile zipFile = new ZipFile(zipFilePath.toFile())) {
            return zipFile.stream()
                    .anyMatch(entry -> entry.isDirectory());
        }
    }

    // 新增：TAR.GZ格式文件夹判断
    private boolean hasFolderInTarGz(Path filePath) throws IOException {
        try (InputStream fi = Files.newInputStream(filePath);
             InputStream gi = new GzipCompressorInputStream(fi);
             TarArchiveInputStream ti = new TarArchiveInputStream(gi)) {

            TarArchiveEntry entry;
            while ((entry = ti.getNextTarEntry()) != null) {
                if (entry.isDirectory()) {
                    return true;
                }
            }
            return false;
        }
    }

    // 新增：7Z格式文件夹判断
    private boolean hasFolderIn7z(Path filePath) throws IOException {
        try (SevenZFile szFile = new SevenZFile(filePath.toFile())) {
            SevenZArchiveEntry entry;
            while ((entry = szFile.getNextEntry()) != null) {
                if (entry.isDirectory()) {
                    return true;
                }
            }
            return false;
        }
    }


    // 保持原有解压逻辑结构的ZIP实现
    // 修改后的ZIP解压方法（修复解压问题）
    private File unzipFile(Path sourcePath, String baseDir) throws IOException {
        // 创建目标目录
        Path destRoot = Paths.get(baseDir);
        Files.createDirectories(destRoot);

        try (ZipInputStream zis = new ZipInputStream(
                Files.newInputStream(sourcePath), StandardCharsets.UTF_8)) {

            ZipEntry entry;
            while ((entry = zis.getNextEntry()) != null) {
                try {
                    // 跳过 macOS 系统文件
                    if (entry.getName().startsWith("__MACOSX")) {
                        continue;
                    }

                    // 标准化路径并处理目录标识
                    String entryName = entry.getName()
                            .replace("\\", "/")
                            .replaceFirst("^/+", ""); // 去除开头的斜杠

                    // 检测是否为目录（兼容以'/'结尾的条目）
                    boolean isDirectory = entry.isDirectory() || entry.getName().endsWith("/");

                    // 调整路径：去除顶层目录（假设所有文件在单一顶层目录下）
                    if (entryName.contains("/")) {
                        int firstSlash = entryName.indexOf('/');
                        entryName = entryName.substring(firstSlash + 1);

                        // 若处理后名称为空，则跳过顶层目录条目
                        if (entryName.isEmpty() && isDirectory) {
                            continue;
                        }
                    }

                    Path targetPath = destRoot.resolve(entryName).normalize();
                    validatePathSafetya(targetPath, destRoot);

                    // 处理目录
                    if (isDirectory) {
                        Files.createDirectories(targetPath);
                    }
                    // 处理文件
                    else {
                        if (Files.exists(targetPath)) {
                            LOGGER.warn("文件已存在，跳过覆盖: {}", targetPath);
                            continue;
                        }
                        // 确保父目录存在
                        Files.createDirectories(targetPath.getParent());
                        try (OutputStream os = Files.newOutputStream(targetPath)) {
                            IOUtils.copy(zis, os);
                        }
                    }
                } finally {
                    zis.closeEntry(); // 确保每个条目只关闭一次
                }
            }
        }

        return destRoot.toFile();
    }

    /**
     * 路径安全验证（增强版）
     */
    private void validatePathSafetya(Path targetPath, Path destRoot) throws IOException {
        Path normalizedDest = destRoot.toAbsolutePath().normalize();
        Path normalizedTarget = targetPath.toAbsolutePath().normalize();

        if (!normalizedTarget.startsWith(normalizedDest)) {
            throw new IOException("路径安全验证失败：" + normalizedTarget
                    + " 超出根目录 " + normalizedDest);
        }
    }


    // 新增：TAR.GZ解压实现（保持相同路径逻辑）
    private File unTarGzFile(Path sourcePath, String baseDir) throws IOException {
        Path destRoot = Paths.get(baseDir);
        Files.createDirectories(destRoot);

        try (InputStream fi = Files.newInputStream(sourcePath);
             InputStream gi = new GzipCompressorInputStream(fi);
             TarArchiveInputStream ti = new TarArchiveInputStream(gi)) {

            TarArchiveEntry entry;
            while ((entry = ti.getNextTarEntry()) != null) {
                // 保持相同过滤逻辑
                if (entry.getName().startsWith("__MACOSX/")) {
                    continue;
                }

                Path targetPath = destRoot.resolve(entry.getName()).normalize();
                validatePathSafety(targetPath, destRoot);

                if (entry.isDirectory()) {
                    Files.createDirectories(targetPath);
                } else {
                    Files.createDirectories(targetPath.getParent());
                    try (OutputStream os = Files.newOutputStream(targetPath)) {
                        IOUtils.copy(ti, os);
                    }
                }
            }
        }
        return destRoot.toFile();
    }

    // 新增：7Z解压实现（保持相同路径逻辑）
    private File un7zFile(Path sourcePath, String baseDir) throws IOException {
        Path destRoot = Paths.get(baseDir);
        Files.createDirectories(destRoot);

        try (SevenZFile szFile = new SevenZFile(sourcePath.toFile())) {
            SevenZArchiveEntry entry;
            while ((entry = szFile.getNextEntry()) != null) {
                // 保持相同过滤逻辑
                if (entry.getName().startsWith("__MACOSX/")) {
                    continue;
                }

                Path targetPath = destRoot.resolve(entry.getName()).normalize();
                validatePathSafety(targetPath, destRoot);

                if (entry.isDirectory()) {
                    Files.createDirectories(targetPath);
                } else {
                    Files.createDirectories(targetPath.getParent());
                    try (OutputStream os = Files.newOutputStream(targetPath)) {
                        byte[] buffer = new byte[8192];
                        int len;
                        while ((len = szFile.read(buffer)) > 0) {
                            os.write(buffer, 0, len);
                        }
                    }
                }
            }
        }
        return destRoot.toFile();
    }


//    /**
//     * 解析压缩包条目（支持多种格式）
//     */
//    private List<ZipEntryInfo> parseArchiveEntries(Path compressedFilePath) throws IOException {
//        List<ZipEntryInfo> entries = new ArrayList<>();
//        String suffix = getMatchedSuffix(compressedFilePath);
//
//        switch (suffix) {
//            case "zip":
//                try (ZipInputStream zis = new ZipInputStream(Files.newInputStream(compressedFilePath))) {
//                    ZipEntry entry;
//                    while ((entry = zis.getNextEntry()) != null) {
//                        if (!entry.getName().startsWith("__MACOSX/")) {
//                            entries.add(new ZipEntryInfo(entry.getName(), entry.isDirectory()));
//                        }
//                        zis.closeEntry();
//                    }
//                }
//                break;
//            case "tar":
//            case "tar.gz":
//            case "tar.bz2":
//                try (ArchiveInputStream ais = createArchiveInputStream(compressedFilePath, suffix)) {
//                    ArchiveEntry entry;
//                    while ((entry = ais.getNextEntry()) != null) {
//                        if (!entry.getName().startsWith("__MACOSX/")) {
//                            entries.add(new ZipEntryInfo(entry.getName(), entry.isDirectory()));
//                        }
//                    }
//                }
//                break;
//            case "rar":
//                try (Archive archive = new Archive(compressedFilePath.toFile())) {
//                    FileHeader fileHeader;
//                    while ((fileHeader = archive.nextFileHeader()) != null) {
//                        if (!fileHeader.getFileName().startsWith("__MACOSX/")) {
//                            entries.add(new ZipEntryInfo(fileHeader.getFileName(), fileHeader.isDirectory()));
//                        }
//                    }
//                } catch (RarException e) {
//                    throw new IOException("Failed to parse RAR entries", e);
//                }
//                break;
//            default:
//                throw new IOException("Unsupported format for entry parsing: " + suffix);
//        }
//        return entries;
//    }


    //上面这个方法会覆盖 TODO
//    private File unzipToTemp(Path zipFilePath, String baseDir) throws IOException {
//        // 1. 直接构建目标路径（baseDir/zipName）
//        String zipName = getFileNameWithoutExtension(zipFilePath.getFileName().toString());
//        Path destRoot = Paths.get(baseDir, zipName);
//        Files.createDirectories(destRoot); // 确保目标目录存在
//
//        try (ZipInputStream zis = new ZipInputStream(new FileInputStream(zipFilePath.toFile()))) {
//            ZipEntry entry;
//            while ((entry = zis.getNextEntry()) != null) {
//                if (entry.getName().startsWith("__MACOSX/")) {
//                    zis.closeEntry();
//                    continue;
//                }
//
//                // 2. 直接解压到目标路径（不再嵌套临时目录）
//                Path destPath = destRoot.resolve(entry.getName()).normalize();
//                validatePathSafety(destPath, destRoot); // 确保路径安全
//
//                // 3. 处理目录
//                if (entry.isDirectory()) {
//                    Files.createDirectories(destPath);
//                }
//                // 4. 处理文件
//                else {
//                    // 确保父目录存在
//                    Files.createDirectories(destPath.getParent());
//
//                    // 写入文件（使用缓冲提升性能）
//                    try (BufferedOutputStream bos = new BufferedOutputStream(
//                            Files.newOutputStream(destPath, StandardOpenOption.CREATE_NEW))) {
//                        byte[] buffer = new byte[8192];
//                        int len;
//                        while ((len = zis.read(buffer)) > 0) {
//                            bos.write(buffer, 0, len);
//                        }
//                    }
//                }
//                zis.closeEntry();
//            }
//        }
//        return destRoot.toFile();
//    }


    /**
     * 校验路径安全性（防御路径穿越攻击）
     */
    private void validatePathSafety(Path destPath, Path tempRoot) throws IOException {
        if (!destPath.startsWith(tempRoot)) {
            throw new IOException("检测到非法路径穿越: " + destPath);
        }
    }

    /**
     * 获取无扩展名的文件名
     */
    private String getFileNameWithoutExtension(String fileName) {
        int dotIndex = fileName.lastIndexOf('.');
        return (dotIndex == -1) ? fileName : fileName.substring(0, dotIndex);
    }

    /*******************************************************本地和minio对比开始****************************************************************/
    @Override
    public TsFiles compareDirectories(List<String> dataset, String nodeId, String taskId) {
        // 获取本地文件路径根目录和存储空间名称
        StorageSourceConfig filePathConfig = getStorageConfig("filePath");
        StorageSourceConfig bucketConfig = getStorageConfig("bucketName");

        TsFiles tsFiles = new TsFiles();
        try {
            // 获取 MinIO 和本地文件列表（并行处理）
            List<FileItemResult> fileItemListMinio = Collections.synchronizedList(new ArrayList<>());
            List<FileItemResult> fileItemListLocal = Collections.synchronizedList(new ArrayList<>());

            if (StringUtils.isNoneEmpty(nodeId, taskId)) {
                // 并行查询数据库记录
                tsFilesMapper.selectList(buildTaskQuery(nodeId, taskId)).parallelStream()
                        .forEach(tsFile -> {
                            try {
                                processFileLists(tsFile, fileItemListMinio, fileItemListLocal);
                            } catch (Exception e) {
                                LOGGER.error("通过nodeId和taskId 获取本地和minio列表失败: {}", e.getMessage(), e);
                                e.printStackTrace();
                            }
                        });
            } else {
                dataset.parallelStream()
                        .map(tsFilesMapper::selectById)
                        .filter(Objects::nonNull)
                        .forEach(tsFile -> {
                            try {
                                processFileLists(tsFile, fileItemListMinio, fileItemListLocal);
                            } catch (Exception e) {
                                LOGGER.error("通过ID 获取本地和minio列表失败: {}", e.getMessage(), e);
                                e.printStackTrace();
                            }
                        });
            }

            // 去重逻辑（并行去重）
            deduplicateFileList(fileItemListMinio);
            deduplicateFileList(fileItemListLocal);

            // 对比文件（并行计算 MD5）
            tsFiles = compareFiles(fileItemListMinio, fileItemListLocal,
                    filePathConfig.getValue(), bucketConfig.getValue(), nodeId, taskId);

            LOGGER.info("MinIO文件列表: {}", fileItemListMinio);
            LOGGER.info("本地文件列表: {}", fileItemListLocal);
        } catch (Exception e) {
            LOGGER.error("目录对比失败: {}", e.getMessage(), e);
        }
        return tsFiles;
    }


    // 辅助方法：获取存储配置
    private StorageSourceConfig getStorageConfig(String name) {
        return storageSourceConfigMapper.selectOne(
                new QueryWrapper<StorageSourceConfig>().eq("name", name)
        );
    }

    // 辅助方法：构建查询条件
    private QueryWrapper<TsFiles> buildTaskQuery(String nodeId, String taskId) {
        return new QueryWrapper<TsFiles>()
                .eq("task_id", taskId)
                .eq("node_id", nodeId);
        //.and(wrapper -> wrapper.eq("work_path", "/").or().eq("backup_path", "/"));
    }

    // 辅助方法：处理文件列表（并行安全）
    private void processFileLists(TsFiles tsFile, List<FileItemResult> minioList, List<FileItemResult> localList) throws Exception {
        processFileList(tsFile.getBackupPath(), tsFile.getFileName(), "minio", minioList, tsFile.getIsFile());
        processFileList(tsFile.getWorkPath(), tsFile.getFileName(), "local", localList, tsFile.getIsFile());
    }

    // 辅助方法：获取文件列表并添加到集合
    private void processFileList(String path, String fileName, String storageKey, List<FileItemResult> targetList, String isFile) throws Exception {
        if (StringUtils.isNotEmpty(path)) {
            AbstractBaseFileService<?> service = storageSourceContext.getByStorageKey(storageKey);
            if (isFile.equals("FOLDER") && storageKey.equals("minio")) {
                List<FileItemResult> files = service.fileListData(path, fileName + "/");
                if (files != null) {
                    // 对每个文件的路径进行规范化
                    files.forEach(file -> {
                        String normalizedPath = ensurePathFormat(file.getPath());
                        file.setPath(normalizedPath);
                    });
                    // 同步添加（线程安全）
                    synchronized (targetList) {
                        targetList.addAll(files);
                    }
                }
            } else {
                List<FileItemResult> files = service.fileListData(path, fileName);
                if (files != null) {
                    // 对每个文件的路径进行规范化
                    files.forEach(file -> {
                        String normalizedPath = ensurePathFormat(file.getPath());
                        file.setPath(normalizedPath);
                    });
                    // 同步添加（线程安全）
                    synchronized (targetList) {
                        targetList.addAll(files);
                    }
                }
            }
        }
    }


    /**
     * 确保路径格式为以 "/" 开头和结尾（例如 "/data/test/"）
     * 若路径为空或非法，返回根路径 "/"
     */
    private String ensurePathFormat(String path) {
        if (StringUtils.isBlank(path)) {
            return "/";
        }
        // 统一替换反斜杠为斜杠（兼容 Windows 路径）
        String normalized = path.replaceAll("\\\\", "/");
        // 确保以 "/" 开头
        if (!normalized.startsWith("/")) {
            normalized = "/" + normalized;
        }
        // 确保以 "/" 结尾
        if (!normalized.endsWith("/")) {
            normalized += "/";
        }
        return normalized;
    }

    // 辅助方法：去重文件列表（并行安全）
    private void deduplicateFileList(List<FileItemResult> fileList) {
        ConcurrentMap<String, Boolean> seenKeys = new ConcurrentHashMap<>();
        fileList.removeIf(file -> {
            String key = normalizePath(file.getPath()) + file.getName();
            return seenKeys.putIfAbsent(key, Boolean.TRUE) != null;
        });
    }

    public TsFiles compareFiles(List<FileItemResult> minioFiles,
                                List<FileItemResult> localFiles,
                                String filePath,
                                String bucketName, String nodeId, String taskId) {
        TsFiles result = new TsFiles();

        // 使用并行流构建 Map（路径 + 文件名 -> 文件）
        ConcurrentMap<String, FileItemResult> minioMap = minioFiles.parallelStream()
                .collect(Collectors.toConcurrentMap(
                        file -> generateMapKey(file),
                        file -> enrichFileMetadata(file, "backup_path", nodeId, taskId)
                ));

        ConcurrentMap<String, FileItemResult> localMap = localFiles.parallelStream()
                .collect(Collectors.toConcurrentMap(
                        file -> generateMapKey(file),
                        file -> enrichFileMetadata(file, "work_path", nodeId, taskId)
                ));

        // 检查独有文件（并行过滤）
        List<FileItemResult> localOnly = localFiles.parallelStream()
                .filter(file -> !minioMap.containsKey(generateMapKey(file)))
                .map(file -> formatFileTime(file)) // 格式化时间字段
                .collect(Collectors.toList());

        List<FileItemResult> minioOnly = minioFiles.parallelStream()
                .filter(file -> !localMap.containsKey(generateMapKey(file)))
                .map(file -> formatFileTime(file)) // 格式化时间字段
                .collect(Collectors.toList());

        // 检查不一致文件（并行处理）
        List<FileItemResult> md5Mismatches = Collections.synchronizedList(new ArrayList<>());
        List<String> sizeMismatches = Collections.synchronizedList(new ArrayList<>());

        minioFiles.parallelStream().forEach(minioFile -> {
            String key = generateMapKey(minioFile);
            if (localMap.containsKey(key)) {
                FileItemResult localFile = localMap.get(key);
                checkFileConsistency(minioFile, localFile, filePath, bucketName, sizeMismatches, md5Mismatches);
            }
        });

        // 设置结果
        result.setLocalOnlyFiles(localOnly);
        result.setMinioOnlyFiles(minioOnly);
        result.setMd5MismatchedFiles(md5Mismatches);

        LOGGER.info("大小不一致文件: {}", sizeMismatches);
        LOGGER.info("MD5不一致文件: {}", md5Mismatches);

        return result;
    }

    // 格式化 FileItemResult 中的时间字段
    private FileItemResult formatFileTime(FileItemResult file) {
        // 假设 time 字段是 Date 类型
        Date time = file.getTime();
        if (time != null) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String formattedDate = sdf.format(time);

            // 设置格式化后的时间到 file 中
            file.setFormattedTime(formattedDate);
        }
        return file;
    }

    // 辅助方法：生成 Map Key
    private String generateMapKey(FileItemResult file) {
        return normalizePath(file.getPath()) + file.getName();
    }

    // 辅助方法：丰富文件元数据
    private FileItemResult enrichFileMetadata(FileItemResult file, String pathType, String nodeId, String taskId) {
        // 记录查询条件的值
        String fileName = file.getName();
        String filePath = normalizePath(file.getPath());
        LOGGER.info("查询条件 - fileName: {}, pathType: {}, filePath: {}", fileName, filePath, pathType);
        TsFiles dbRecord = tsFilesMapper.selectOne(
                new QueryWrapper<TsFiles>()
                        .eq("file_name", file.getName())
                        .eq("node_id", nodeId)
                        .eq("task_id", taskId)
                        .eq(pathType, normalizePath(file.getPath()))
        );
        if (dbRecord != null) {
            file.setId(dbRecord.getId());
        } else {
            LOGGER.error("未找到匹配的记录，fileName: {}, pathType: {}, filePath: {}", fileName, filePath, pathType);
        }
        return file;
    }

    // 辅助方法：检查文件一致性
    private void checkFileConsistency(FileItemResult minioFile,
                                      FileItemResult localFile,
                                      String basePath,
                                      String bucketName,
                                      List<String> sizeMismatches,
                                      List<FileItemResult> md5Mismatches) {
        File localFileObj = new File(basePath + minioFile.getPath(), localFile.getName());
        if (localFileObj.isDirectory()) {
            LOGGER.warn("跳过文件夹: {}", localFileObj.getAbsolutePath());
            return;
        }

        // 检查文件大小
        if (!isSizeMatch(minioFile, localFile)) {
            sizeMismatches.add(minioFile.getName());
            return; // 大小不匹配时无需检查 MD5
        }

        // 检查 MD5
        try {
            String localMD5 = calculateMD5(new FileInputStream(localFileObj));
            String minioMD5 = getMinioMD5(minioFile, bucketName);

            if (StringUtils.isNoneEmpty(localMD5, minioMD5) && !localMD5.equals(minioMD5)) {
                minioFile.setLocatMd5(localMD5);
                minioFile.setMinioMd5(minioMD5);
                md5Mismatches.add(minioFile);
            }
        } catch (Exception e) {
            LOGGER.error("MD5计算失败: {} - {}", minioFile.getName(), e.getMessage());
        }
    }

    // 辅助方法：检查文件大小是否匹配
    private boolean isSizeMatch(FileItemResult minioFile, FileItemResult localFile) {
        return minioFile.getSize() != null
                && localFile.getSize() != null
                && minioFile.getSize().equals(localFile.getSize());
    }

    // 辅助方法：获取 MinIO 文件 MD5
    private String getMinioMD5(FileItemResult file, String bucketName) {
        AbstractBaseFileService<?> minioService = storageSourceContext.getByStorageKey("minio");
        String key = StringUtils.concat(file.getPath(), file.getName());

        try {
            if (file.getSize() <= 5L * 1024 * 1024 * 1024) {
                return minioService.getObjectMetadata(bucketName, key).getETag().replace("\"", "");
            } else {
                try (S3Object s3Object = minioService.getObject(bucketName, key)) {
                    return calculateMD5(s3Object.getObjectContent());
                }
            }
        } catch (Exception e) {
            LOGGER.error("MinIO MD5获取失败: {}", key, e);
            return null;
        }
    }

    // 优化后的 MD5 计算方法（线程安全）
    public String calculateMD5(InputStream inputStream) throws IOException, NoSuchAlgorithmException {
        try (InputStream is = inputStream) { // 确保流关闭
            MessageDigest md = MessageDigest.getInstance("MD5");
            byte[] buffer = new byte[8192];
            int bytesRead;
            while ((bytesRead = is.read(buffer)) != -1) {
                md.update(buffer, 0, bytesRead);
            }
            return Hex.encodeHexString(md.digest());
        }
    }


//    @Override
//    public TsFiles compareDirectories(List<String> dataset, String nodeId, String taskId) {
//        //查询本地文件路径根目录 如 E:\yun
//        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("name", "filePath");
//        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);
//
//        //查询存储空间名称根目录 如test-bucket
//        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
//        queryWrapper1.eq("name", "bucketName");
//        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);
//        // 获取文件列表
//        TsFiles tsFiles = new TsFiles();
//        try {
//            List<FileItemResult> fileItemListMinio = new ArrayList<>();
//            List<FileItemResult> fileItemListLocal = new ArrayList<>();
//            if (StringUtils.isNotEmpty(nodeId) && StringUtils.isNotEmpty(taskId)) {
//                QueryWrapper<TsFiles> queryWrapper2 = new QueryWrapper<>();
//                queryWrapper2.eq("task_id", taskId)
//                        .eq("node_id", nodeId).and(i -> i.eq("work_path", "/").or()
//                        .eq("backup_path", "/"));
//                List<TsFiles> allNodes = tsFilesMapper.selectList(queryWrapper2);
//                for (TsFiles tsFiles1 : allNodes) {
//                    if (tsFiles1 != null) {
//                        // 获取minio文件列表
//                        if (StringUtils.isNotEmpty(tsFiles1.getBackupPath())) {
//                            AbstractBaseFileService<?> fileServiceMinio = storageSourceContext.getByStorageKey("minio");
//                            List<FileItemResult> minioFiles = fileServiceMinio.fileListData(tsFiles1.getBackupPath(), tsFiles1.getFileName());
//                            // 将当前文件列表添加到 fileItemListMinio
//                            if (minioFiles != null) {
//                                fileItemListMinio.addAll(minioFiles);
//                            }
//                        }
//
//                        // 获取本地文件列表
//                        if (StringUtils.isNotEmpty(tsFiles1.getWorkPath())) {
//                            AbstractBaseFileService<?> fileServiceLocal = storageSourceContext.getByStorageKey("local");
//                            List<FileItemResult> localFiles = fileServiceLocal.fileListData(tsFiles1.getWorkPath(), tsFiles1.getFileName());
//
//                            // 将当前文件列表添加到 fileItemListLocal
//                            if (localFiles != null) {
//                                fileItemListLocal.addAll(localFiles);
//                            }
//                        }
//                    }
//                }
//
//            } else {
//                for (String id : dataset) {
//                    TsFiles tsFiles1 = tsFilesMapper.selectById(id);
//                    if (tsFiles1 != null) {
//                        // 获取minio文件列表
//                        if (StringUtils.isNotEmpty(tsFiles1.getBackupPath())) {
//                            AbstractBaseFileService<?> fileServiceMinio = storageSourceContext.getByStorageKey("minio");
//                            List<FileItemResult> minioFiles = fileServiceMinio.fileListData(tsFiles1.getBackupPath(), tsFiles1.getFileName());
//                            // 将当前文件列表添加到 fileItemListMinio
//                            if (minioFiles != null) {
//                                fileItemListMinio.addAll(minioFiles);
//                            }
//                        }
//
//                        // 获取本地文件列表
//                        if (StringUtils.isNotEmpty(tsFiles1.getWorkPath())) {
//                            AbstractBaseFileService<?> fileServiceLocal = storageSourceContext.getByStorageKey("local");
//                            List<FileItemResult> localFiles = fileServiceLocal.fileListData(tsFiles1.getWorkPath(), tsFiles1.getFileName());
//
//                            // 将当前文件列表添加到 fileItemListLocal
//                            if (localFiles != null) {
//                                fileItemListLocal.addAll(localFiles);
//                            }
//                        }
//                    }
//                }
//            }
//
//            //去除重复的
//            Set<String> seenMinioPaths = new HashSet<>(); // 用于记录已经处理过的 path + name
//            Iterator<FileItemResult> iteratorMinio = fileItemListMinio.iterator();
//            while (iteratorMinio.hasNext()) {
//                FileItemResult fileItemResult = iteratorMinio.next();
//                String path = normalizePath(fileItemResult.getPath());
//                String name = fileItemResult.getName();
//                String uniqueKey = path + name; // 唯一标识
//                if (seenMinioPaths.contains(uniqueKey)) {
//                    // 如果已经存在，则移除当前项
//                    iteratorMinio.remove();
//                } else {
//                    // 如果不存在，则添加到已处理集合中，并更新 path
//                    seenMinioPaths.add(uniqueKey);
//                    fileItemResult.setPath(path);
//                }
//            }
//
//            //去除重复的
//            Set<String> seenLocalPaths = new HashSet<>(); // 用于记录已经处理过的 path + name
//            Iterator<FileItemResult> iteratorLocal = fileItemListLocal.iterator();
//            while (iteratorLocal.hasNext()) {
//                FileItemResult fileItemResult = iteratorLocal.next();
//                String path = normalizePath(fileItemResult.getPath());
//                String name = fileItemResult.getName();
//                String uniqueKey = path + name; // 唯一标识
//
//                if (seenLocalPaths.contains(uniqueKey)) {
//                    // 如果已经存在，则移除当前项
//                    iteratorLocal.remove();
//                } else {
//                    // 如果不存在，则添加到已处理集合中，并更新 path
//                    seenLocalPaths.add(uniqueKey);
//                    fileItemResult.setPath(path);
//                }
//            }
//
//            // 对比文件
//            tsFiles = compareFiles(fileItemListMinio, fileItemListLocal, storageSourceConfig.getValue(), storageSourceConfig1.getValue());
//            LOGGER.info("minio文件 " + fileItemListMinio);
//            LOGGER.info("本地文件 " + fileItemListLocal);
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return tsFiles;
//    }
//
//
//
//
//    public TsFiles compareFiles(List<FileItemResult> fileItemListMinio, List<FileItemResult> fileItemListLocal, String filePath, String bucketName) {
//        TsFiles tsFiles = new TsFiles();
//
//        // 将 MinIO 文件列表转换为 Map（文件名 -> 文件信息）
//        Map<String, FileItemResult> minioFileMap = new HashMap<>();
//        for (FileItemResult file : fileItemListMinio) {
//            // 处理路径格式
//            String path = normalizePath(file.getPath());
//            QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
//            queryWrapper.eq("file_name", file.getName());
//            queryWrapper.eq("backup_path", path);
//            TsFiles tsFiles1 = tsFilesMapper.selectOne(queryWrapper);
//            if (tsFiles1 != null) {
//                file.setId(tsFiles1.getId());
//
//            }
//
//            minioFileMap.put(path + file.getName(), file);
//        }
//
//        // 将本地文件列表转换为 Map（文件名 -> 文件信息）
//        Map<String, FileItemResult> localFileMap = new HashMap<>();
//        for (FileItemResult file : fileItemListLocal) {
//            // 处理路径格式
//            String path = normalizePath(file.getPath());
//            QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
//            queryWrapper.eq("file_name", file.getName());
//            queryWrapper.eq("work_path", path);
//            TsFiles tsFiles1 = tsFilesMapper.selectOne(queryWrapper);
//            if (tsFiles1 != null) {
//                file.setId(tsFiles1.getId());
//
//            }
//            localFileMap.put(path + file.getName(), file);
//        }
//
//        // 检查本地有而 MinIO 没有的文件
//        List<FileItemResult> localOnlyFiles = new ArrayList<>();
//        for (FileItemResult localFile : fileItemListLocal) {
//            // 处理路径格式
//            String path = normalizePath(localFile.getPath());
//            if (!minioFileMap.containsKey(path + localFile.getName())) {
//                localOnlyFiles.add(localFile);
//            }
//        }
//        LOGGER.info("本地有而 MinIO 没有的文件：" + localOnlyFiles);
//
//        // 检查 MinIO 有而本地没有的文件
//        List<FileItemResult> minioOnlyFiles = new ArrayList<>();
//        for (FileItemResult minioFile : fileItemListMinio) {
//            String path = normalizePath(minioFile.getPath());
//            if (!localFileMap.containsKey(path + minioFile.getName())) {
//                minioOnlyFiles.add(minioFile);
//            }
//        }
//        LOGGER.info("MinIO 有而本地没有的文件：" + minioOnlyFiles);
//
//        // 检查文件大小不一致的文件
//        List<String> sizeMismatchedFiles = new ArrayList<>();
//
//        // 检查 MD5 不一致的文件
//        List<FileItemResult> md5MismatchedFiles = new ArrayList<>();
//
//        // 遍历 MinIO 文件
//        for (FileItemResult minioFile : fileItemListMinio) {
//            String path = normalizePath(minioFile.getPath());
//            // 检查 MinIO 文件是否在本地存在
//            if (localFileMap.containsKey(path + minioFile.getName())) {
//                FileItemResult localFile = localFileMap.get(path + minioFile.getName());
//                String localBasePath = filePath + minioFile.getPath();
//                // 检查文件是否为文件夹，如果是文件夹，跳过
//                File localFilePath = new File(localBasePath + localFile.getName());
//                if (localFilePath.isDirectory()) {
//                    LOGGER.error("路径指向的是一个文件夹，跳过该文件夹：" + localFilePath.getAbsolutePath());
//                    continue; // 跳过文件夹
//                }
//
//                // 检查文件大小是否匹配
//                if (minioFile.getSize() != null && localFile.getSize() != null && minioFile.getSize().longValue() != localFile.getSize().longValue()) {
//                    sizeMismatchedFiles.add(minioFile.getName());
//                }
//                // 如果文件大小一致，进一步检查 MD5
//                try {
//                    // 计算本地文件的 MD5
//                    String localMD5 = calculateMD5(new FileInputStream(localFilePath));
//                    if (localMD5 == null || localMD5.isEmpty()) {
//                        continue;
//                    }
//
//                    // 获取 MinIO 文件的 MD5
//                    String minioMD5;
//                    AbstractBaseFileService<?> fileServiceMinio = storageSourceContext.getByStorageKey("minio");
//                    String key = StringUtils.concat(minioFile.getPath(), minioFile.getName()); // 构造文件的 key
//                    if (minioFile.getSize() <= 5 * 1024 * 1024 * 1024L) { // 小于 5GB，使用 ETag
//                        ObjectMetadata metadata = fileServiceMinio.getObjectMetadata(bucketName, key);
//                        minioMD5 = metadata.getETag().replace("\"", ""); // 去除 ETag 的引号
//                    } else { // 大于 5GB，手动计算 MD5
//                        S3Object s3Object = fileServiceMinio.getObject(bucketName, key);
//                        minioMD5 = calculateMD5(s3Object.getObjectContent());
//                    }
//
//                    // 比较 MD5
//                    if (!minioMD5.equals(localMD5)) {
//                        minioFile.setLocatMd5(localMD5);
//                        minioFile.setMinioMd5(minioMD5);
//                        md5MismatchedFiles.add(minioFile);
//                    }
//                } catch (Exception e) {
//                    LOGGER.error("计算 MD5 失败：" + e.getMessage());
//                }
//
//            } else {
//                // 如果 MinIO 中没有该文件，本地有的情况下，跳过该文件
//                LOGGER.info("MinIO 中没有该文件，本地有文件，跳过对比: " + minioFile.getName());
//            }
//        }
//        tsFiles.setLocalOnlyFiles(localOnlyFiles);
//        tsFiles.setMinioOnlyFiles(minioOnlyFiles);
//        tsFiles.setMd5MismatchedFiles(md5MismatchedFiles);
//        LOGGER.info("文件大小不一致的文件：" + sizeMismatchedFiles);
//        LOGGER.info("文件大小一致但 MD5 不一致的文件：" + md5MismatchedFiles);
//        return tsFiles;
//
//    }
//
//
//    public String calculateMD5(InputStream inputStream) throws Exception {
//        MessageDigest md = MessageDigest.getInstance("MD5");
//        byte[] buffer = new byte[8192];
//        int bytesRead;
//        while ((bytesRead = inputStream.read(buffer)) != -1) {
//            md.update(buffer, 0, bytesRead);
//        }
//        byte[] digest = md.digest();
//        StringBuilder sb = new StringBuilder();
//        for (byte b : digest) {
//            sb.append(String.format("%02x", b));
//        }
//        return sb.toString();
//    }


/*******************************************************本地和minio对比结束****************************************************************/


    /******************************************************对比结束以后开始同步***************************************************************************************/
    /**********************************
     * 用途说明: 将文件上传到备份空间
     * 参数说明  parameter 数据集合
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回成功或者失败
     ***********************************/
    @Override
    public Boolean uploadToBackup(Parameter parameter) {
        Boolean value = false;

        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);

        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);
        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("name", "bucketName");
        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);

        List<ParameterList> parameterLists = parameter.getParameterLists();

        //循环一条一条上传
        for (ParameterList parameterListData : parameterLists) {
            if ("FOLDER".equals(parameterListData.getType())) {
                String name = parameterListData.getName();
                String path = parameterListData.getPath();
                // 处理路径格式
                path = normalizePath(path);


                String localPath = storageSourceConfig.getValue() + path;//localPath是本地的路径
                String minioPath = path;//// MinIO上传目标路径
                // 使用Java NIO的Path类来构造压缩文件的完整路径，存储在localPath目录下
                // 查询当前文件记录
                TsFiles currentFile = getFileRecord(name, path);
                if (currentFile == null) return false;

                // 递归创建父文件夹
                boolean parentsCreated = createParentFolders(currentFile.getParentId());
                if (!parentsCreated) {
                    LOGGER.error("父文件夹创建失败");
                    return false;
                }

                //新增节点的时候 创建文件夹
                NewFolderRequest newFolderRequest = new NewFolderRequest();
                newFolderRequest.setName(currentFile.getFileName());//新建的文件夹名称,示例值(/a/b/c)
                newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
                newFolderRequest.setPath(minioPath);//请求路径,示例值(/)
                newFolderRequest.setStorageKey("minio");//存储源 key,示例值(local minio)
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
                boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
                LOGGER.info("同步本地到minio文件夹路径加名称" + newFolderRequest.getPath() + newFolderRequest.getName());

                //如果文件夹创建成功
                if (flag) {
                    currentFile.setUpdateTime(currentTime);
                    currentFile.setBackupPath(minioPath);
                    int valueAdded = tsFilesMapper.updateById(currentFile);
                    if (valueAdded == 1) {
                        LOGGER.info("同步本地到minio,minio文件夹创建成功,表结构修改成功。");
                        value = true;
                    } else {
                        LOGGER.error("同步本地到minio,minio文件夹创建成功，表结构修改失败");
                        value = false;
                    }

                } else {
                    LOGGER.error("同步本地到minio,minio文件夹创建失败");
                    value = false;
                }
            } else {

                String name = parameterListData.getName();
                String path = parameterListData.getPath();
                String size = parameterListData.getSize();
                // 处理路径格式
                path = normalizePath(path);

                String localPath = storageSourceConfig.getValue() + path;//localPath是本地的路径
                String minioPath = path;//// MinIO上传目标路径

                // 使用Java NIO的Path类来构造压缩文件的完整路径，存储在localPath目录下
                Path FilePath = Paths.get(localPath, name);
                // 4. 上传压缩文件到MinIO
                File zipFile = FilePath.toFile(); // 获取本地文件

                // 查询当前文件记录
                TsFiles currentFile = getFileRecord(name, path);
                if (currentFile == null) return false;

                // 递归创建父文件夹
                boolean parentsCreated = createParentFolders(currentFile.getParentId());
                if (!parentsCreated) {
                    LOGGER.error("父文件夹创建失败");
                    return false;
                }
                //把本地文件上传到minio
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("minio");
                boolean flag1 = fileService.UploadFiles(storageSourceConfig1.getValue(), minioPath + name, zipFile);
                LOGGER.info("同步本地到minio文件路径加名称" + storageSourceConfig1.getValue() + minioPath + name);

                //如果上传成功  修改 表结构
                if (flag1) {
                    currentFile.setUpdateTime(currentTime);
                    currentFile.setBackupPath(minioPath);
                    int valueAdded = tsFilesMapper.updateById(currentFile);
                    if (valueAdded == 1) {
                        LOGGER.info("同步本地到minio,minio文件创建成功,表结构修改成功。");
                        value = true;
                    } else {
                        LOGGER.error("同步本地到minio,minio文件创建成功，表结构修改失败");
                        value = false;
                    }
                } else {
                    LOGGER.error("同步本地到minio,minio文件创建失败");
                    value = false;
                }
            }
        }
        return value;
    }

    // 规范化路径格式
    private String normalizePath(String path) {
        if (!path.startsWith("/")) path = "/" + path;
        if (!path.endsWith("/")) path += "/";
        return path.replace("//", "/");
    }

    // 查询文件记录
    private TsFiles getFileRecord(String name, String path) {
        QueryWrapper<TsFiles> query = new QueryWrapper<>();
        query.eq("file_name", name)
                .eq("work_path", path);
        return tsFilesMapper.selectOne(query);
    }

    // 递归创建minio父文件夹
    private boolean createParentFolders(String parentId) {
        TsFiles parentFolder = tsFilesMapper.selectById(parentId);
        if (parentFolder == null || "00".equals(parentId)) {
            return true; // 到达根目录
        }

        // 先递归处理上级
        boolean ancestorsCreated = createParentFolders(parentFolder.getParentId());
        if (!ancestorsCreated) return false;

        // 检查当前文件夹是否已创建
        if (StringUtils.isEmpty(parentFolder.getBackupPath())) {
            boolean folderCreated = createMinioFolder(parentFolder);
            if (!folderCreated) return false;
        }
        return true;
    }

    // 创建MinIO文件夹
    private boolean createMinioFolder(TsFiles folder) {
        try {
            // 设置当前时间
            LocalDateTime now = LocalDateTime.now();
            // 转换为 Timestamp
            Timestamp currentTime = Timestamp.valueOf(now);
            AbstractBaseFileService<?> minioService = storageSourceContext.getByStorageKey("minio");
            //String folderPath = folder.getWorkPath() + folder.getFileName() + "/";
            String folderPath = folder.getWorkPath() + folder.getFileName();

            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(folder.getFileName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(folder.getWorkPath());//请求路径,示例值(/)
            newFolderRequest.setStorageKey("minio");//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {
                folder.setUpdateTime(currentTime);
                folder.setBackupPath(folder.getWorkPath());
                tsFilesMapper.updateById(folder);
                LOGGER.info("创建文件夹成功：{}", folderPath);
                return true;
            }
            return false;
        } catch (Exception e) {
            LOGGER.error("创建文件夹失败：{}", folder.getFileName(), e);
            return false;
        }
    }


    /**********************************
     * 用途说明: 从备份空间下载到工作空间
     * 参数说明  parameter 数据集合
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回成功或者失败
     ***********************************/
    @Override
    public Boolean downloadToLocal(Parameter parameter) {
        Boolean value = true;

        // 设置当前时间
        LocalDateTime now = LocalDateTime.now();
        // 转换为 Timestamp
        Timestamp currentTime = Timestamp.valueOf(now);

        //上传人是当前登录人
        UsernamePasswordAuthenticationToken authentication = (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();

        QueryWrapper<StorageSourceConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", "filePath");
        StorageSourceConfig storageSourceConfig = storageSourceConfigMapper.selectOne(queryWrapper);


        QueryWrapper<StorageSourceConfig> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("name", "bucketName");
        StorageSourceConfig storageSourceConfig1 = storageSourceConfigMapper.selectOne(queryWrapper1);
        List<ParameterList> parameterLists = parameter.getParameterLists();
        for (ParameterList parameterListData : parameterLists) {
            //创建文件夹
            if ("FOLDER".equals(parameterListData.getType())) {
                String name = parameterListData.getName();
                String path = parameterListData.getPath();
                // 处理路径格式
                path = normalizePath(path);

                //通过名称和minio路径查询表结构的数据
                QueryWrapper<TsFiles> queryWrapperFile = new QueryWrapper<>();
                queryWrapperFile.eq("file_name", name);
                queryWrapperFile.eq("backup_path", path);
                TsFiles tsFiles = tsFilesMapper.selectOne(queryWrapperFile);
                if (tsFiles == null) {
                    return false;
                }
                LOGGER.info("同步minio到本地,查询的文件夹对象", tsFiles);

                // 递归创建父文件夹
                boolean parentsCreated = createLocalParentFolders(tsFiles.getParentId());
                if (!parentsCreated) {
                    LOGGER.error("父文件夹创建失败");
                    return false;
                }


                //新增节点的时候 创建文件夹
                NewFolderRequest newFolderRequest = new NewFolderRequest();
                newFolderRequest.setName(tsFiles.getFileName());//新建的文件夹名称,示例值(/a/b/c)
                newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
                newFolderRequest.setPath(path);//请求路径,示例值(/)
                newFolderRequest.setStorageKey("local");//存储源 key,示例值(local minio)
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
                boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
                LOGGER.info("同步minio到本地文件夹路径加名称" + newFolderRequest.getPath() + newFolderRequest.getName());

                //如果文件夹创建成功
                if (flag) {
                    tsFiles.setUpdateTime(currentTime);
                    tsFiles.setWorkPath(path);
                    int valueAdded = tsFilesMapper.updateById(tsFiles);
                    if (valueAdded == 1) {
                        LOGGER.info("同步minio到本地,本地文件夹创建成功,表结构修改成功。");
                        value = true;
                    } else {
                        LOGGER.error("同步minio到本地,本地文件夹创建成功，表结构修改失败");
                        value = false;
                    }

                } else {
                    LOGGER.error("同步minio到本地,本地文件夹创建失败");
                    value = false;
                }

            } else {
                //如果是文件
                String name = parameterListData.getName();
                String path = parameterListData.getPath();
                String size = parameterListData.getSize();
                // 处理路径格式
                path = normalizePath(path);

                //通过名称和minio路径查询表结构的数据
                QueryWrapper<TsFiles> queryWrapperFile = new QueryWrapper<>();
                queryWrapperFile.eq("file_name", name);
                queryWrapperFile.eq("backup_path", path);
                TsFiles tsFiles = tsFilesMapper.selectOne(queryWrapperFile);
                if (tsFiles == null) {
                    return false;
                }
                // 递归创建父文件夹
                boolean parentsCreated = createLocalParentFolders(tsFiles.getParentId());
                if (!parentsCreated) {
                    LOGGER.error("父文件夹创建失败");
                    return false;
                }

                String localPath = storageSourceConfig.getValue() + path + name;//localPath是本地的路径
                String minioPath = path + name;//// MinIO上传目标路径

                FileOutputStream outputStream = null;   // 输出流
                S3ObjectInputStream inputStream = null; // 输入流
                S3Object s3Object = null;
                try {
                    AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("minio");
                    // 获取 MinIO 文件对象
                    s3Object = fileService.getObject(storageSourceConfig1.getValue(), minioPath);
                    // 获取文件输入流
                    inputStream = s3Object.getObjectContent();

                    // 创建本地文件
                    File localFile = new File(localPath);
                    if (!localFile.getParentFile().exists()) {
                        localFile.getParentFile().mkdirs(); // 创建父目录
                    }

                    // 将文件内容写入本地文件
                    outputStream = new FileOutputStream(localFile);
                    IOUtils.copy(inputStream, outputStream);
                    //文件下载成功以后 修改表结构
                    tsFiles.setUpdateTime(currentTime);
                    tsFiles.setWorkPath(path);
                    int valueAdded = tsFilesMapper.updateById(tsFiles);
                    if (valueAdded == 1) {
                        LOGGER.info("同步minio到本地,本地文件创建成功,表结构修改成功。");
                        LOGGER.info("文件下载成功：" + localPath);
                        value = true;
                    } else {
                        LOGGER.error("同步minio到本地,本地文件创建成功，表结构修改失败");
                        value = false;
                    }
                } catch (Exception e) {
                    LOGGER.error("文件下载失败：" + e.getMessage());
                } finally {
                    // 关闭流
                    if (inputStream != null) {
                        try {
                            inputStream.close();
                        } catch (IOException e) {
                            LOGGER.error("关闭输入流失败：" + e.getMessage());
                        }
                    }
                    if (outputStream != null) {
                        try {
                            outputStream.close();
                        } catch (IOException e) {
                            LOGGER.error("关闭输出流失败：" + e.getMessage());
                        }
                    }
                    if (s3Object != null) {
                        try {
                            s3Object.close();
                        } catch (IOException e) {
                            LOGGER.error("关闭 S3Object 失败：" + e.getMessage());
                        }
                    }
                }
            }
        }
        return value;
    }


    // 递归创建本地父文件夹
    private boolean createLocalParentFolders(String parentId) {
        TsFiles parentFolder = tsFilesMapper.selectById(parentId);
        if (parentFolder == null || "00".equals(parentId)) {
            return true; // 到达根目录
        }

        // 先递归处理上级
        boolean ancestorsCreated = createLocalParentFolders(parentFolder.getParentId());
        if (!ancestorsCreated) return false;

        // 检查当前文件夹是否已创建
        if (StringUtils.isEmpty(parentFolder.getWorkPath())) {
            boolean folderCreated = createLocalFolder(parentFolder);
            if (!folderCreated) return false;
        }
        return true;
    }

    // 创建本地文件夹
    private boolean createLocalFolder(TsFiles folder) {
        try {
            // 设置当前时间
            LocalDateTime now = LocalDateTime.now();
            // 转换为 Timestamp
            Timestamp currentTime = Timestamp.valueOf(now);
            //String folderPath = folder.getWorkPath() + folder.getFileName() + "/";
            String folderPath = folder.getBackupPath() + folder.getFileName();

            //新增节点的时候 创建文件夹
            NewFolderRequest newFolderRequest = new NewFolderRequest();
            newFolderRequest.setName(folder.getFileName());//新建的文件夹名称,示例值(/a/b/c)
            newFolderRequest.setPassword("");//文件夹密码, 如果文件夹需要密码才能访问，则支持请求密码,示例值(123456)
            newFolderRequest.setPath(folder.getBackupPath());//请求路径,示例值(/)
            newFolderRequest.setStorageKey("local");//存储源 key,示例值(local minio)
            AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey(newFolderRequest.getStorageKey());
            boolean flag = fileService.newFolder(newFolderRequest.getPath(), newFolderRequest.getName());
            if (flag) {
                folder.setUpdateTime(currentTime);
                folder.setWorkPath(folder.getBackupPath());
                tsFilesMapper.updateById(folder);
                LOGGER.info("创建文件夹成功：{}", folderPath);
                return true;
            }
            return false;
        } catch (Exception e) {
            LOGGER.error("创建文件夹失败：{}", folder.getFileName(), e);
            return false;
        }
    }


    // 辅助方法：关闭资源时，避免抛出异常
    private void closeQuietly(AutoCloseable resource) {
        if (resource != null) {
            try {
                resource.close();
            } catch (Exception e) {
                System.err.println("关闭资源时出现异常：" + e.getMessage());
            }
        }
    }

    /*********************************************************************************************************************************************/


    /**********************************************************移动文件或者文件夹结束***********************************************************************************/
    /**
     * 移动文件或文件夹
     * 参数说明 newPath 新路径
     * 参数说明 oldpaths 原路径
     * 参数说明 ParentId 父ID
     * 参数说明 newFileName
     * 参数说明 Rename 重命名的文件名称
     * 参数说明 type 覆盖还是重命名  0 1
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String moveFileFolder(MoveCopyFileFolderRequest reques) throws IOException {
        // 1. 获取存储根路径
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(
                new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        Path rootPath = Paths.get(config.getValue());

        String newPath = reques.getNewPath();
        String oldpaths = reques.getOldpaths();
        String parentId = reques.getParentId();
        String rename = reques.getRename();
        String type = reques.getType();
        String[] newFileNames = reques.getNewFileName().split(",");
        // 数组转集合
        List<String> newFileNameList = Arrays.asList(newFileNames);
        // 2. 构建目标父目录的物理路径（如 E:\yun\22）
        Path targetParentPhysicalPath = buildPhysicalPath(newPath, rootPath);

        // 3. 循环处理所有选中的文件/文件夹
        for (String fileName : newFileNameList) {
            // 构建原文件物理路径（如 E:\yun\11\file.txt）
            Path sourcePath = buildPhysicalPath(oldpaths + "/" + fileName, rootPath);
            if (!Files.exists(sourcePath)) {
                throw new IOException("源文件不存在: " + sourcePath);
            }

            // ==== 新增关键校验 ====
            // 1. 禁止移动到自身
            if (sourcePath.equals(targetParentPhysicalPath)) {
                throw new IllegalArgumentException("不能移动到自己所在目录");
            }

            // 2. 禁止文件夹移动到子目录
            if (Files.isDirectory(sourcePath)) {
                if (targetParentPhysicalPath.startsWith(sourcePath)) {
                    throw new IllegalArgumentException("禁止将文件夹移动到自己内部");
                }
            }

            // 生成目标文件名（处理重命名和冲突）
            String targetFileName = generateTargetFileName(targetParentPhysicalPath, fileName, rename, type);
            Path targetPath = targetParentPhysicalPath.resolve(targetFileName);

            // 3. 再次检查自移动
            if (sourcePath.equals(targetPath)) {
                throw new IllegalArgumentException("不能移动到自己当前位置");
            }

            // 执行移动操作
            moveFile(sourcePath, targetPath, type);

            // 更新数据库记录
            updateDatabase(parentId, oldpaths, fileName, newPath, targetFileName, type);
        }
        return "移动成功";
    }


    private String generateTargetFileName(Path targetParentPhysicalPath, String originalName, String rename, String type) {
        if ("1".equals(type)) {
            // 重命名模式：直接使用用户指定名称
            return rename;
        } else {
//            // 覆盖模式：处理文件名冲突（自动添加序号）
//            String baseName = originalName.lastIndexOf('.') != -1 ?
//                    originalName.substring(0, originalName.lastIndexOf('.')) : originalName;
//            String extension = originalName.lastIndexOf('.') != -1 ?
//                    originalName.substring(originalName.lastIndexOf('.')) : "";
//
//            int counter = 1;
//            String newName = originalName;
//            while (Files.exists(targetParentPhysicalPath.resolve(newName))) {
//                newName = baseName + "(" + counter + ")" + extension;
//                counter++;
//            }
            return originalName;
        }
    }

    /**
     * 构建完整的物理路径（基于数据库中的 work_path）
     */
    private Path buildPhysicalPath(String dbPath, Path root) {
        // 统一替换所有斜杠为系统分隔符（兼容不同系统）
        String systemPath = dbPath.replace("/", File.separator)
                .replace("\\", File.separator);

        // 处理根路径特殊情况（如 "/" 或空字符串）
        if (systemPath.equals(File.separator) || systemPath.isEmpty()) {
            return root;
        }

        // 修复正则表达式：使用 Pattern.quote 处理特殊字符
        String separatorRegex = Pattern.quote(File.separator);
        systemPath = systemPath.replaceAll("^" + separatorRegex + "+", "");

        try {
            Path resolvedPath = root.resolve(systemPath).normalize();

            // 二次校验防止路径越狱
            if (!resolvedPath.startsWith(root)) {
                throw new SecurityException("非法路径访问: " + dbPath);
            }
            return resolvedPath;
        } catch (InvalidPathException e) {
            throw new IllegalArgumentException("无效路径格式: " + dbPath, e);
        }
    }


    /**
     * 执行文件移动操作
     */
    private void moveFile(Path sourcePath, Path targetPath, String type) throws IOException {
        if ("0".equals(type)) {
            // 覆盖模式：强制替换目标文件
            Files.move(sourcePath, targetPath, StandardCopyOption.REPLACE_EXISTING);
        } else {
            // 重命名模式：检查目标文件是否存在
            if (Files.exists(targetPath)) {
                throw new IOException("目标文件已存在：" + targetPath);
            }
            Files.move(sourcePath, targetPath);
        }

        // 移动后检查原路径是否为空，若为空则重新创建空文件夹（可选）
        if (isPathEmpty(sourcePath.getParent())) {
            Files.createDirectories(sourcePath.getParent());
        }
    }

    /**
     * 更新数据库中的父ID和路径
     */
    private void updateDatabase(String parentId, String oldpaths, String fileName,
                                String newPath, String targetFileName, String type) {
        // 1. 构建完整路径
        String oldWorkPath = oldpaths.endsWith("/") ? oldpaths : oldpaths + "/";
        String newWorkPath = newPath.endsWith("/") ? newPath : newPath + "/";

        // 2. 查询原记录
        TsFiles fileRecord = tsFilesMapper.selectOne(
                new QueryWrapper<TsFiles>().eq("work_path", oldWorkPath)
                        .eq("file_name", fileName));
        if (fileRecord == null) {
            throw new RuntimeException("数据库记录不存在：" + oldWorkPath + fileName);
        }

        // 3. 查询目标位置是否已存在记录
        TsFiles newfileRecord = tsFilesMapper.selectOne(
                new QueryWrapper<TsFiles>().eq("work_path", newWorkPath)
                        .eq("file_name", targetFileName));

        // 4. 根据 type 处理记录
        if ("0".equals(type)) {
            // 覆盖模式
            if (newfileRecord != null) {
                // 更新目标记录
                newfileRecord.setParentId(parentId);
                newfileRecord.setWorkPath(newWorkPath);
                newfileRecord.setFileName(targetFileName);
                newfileRecord.setUpdateTime(new Timestamp(System.currentTimeMillis())); // 更新时间
                tsFilesMapper.updateById(newfileRecord);
                // 删除原记录
                tsFilesMapper.deleteById(fileRecord.getId());
            } else {
                // 插入新记录
                TsFiles newRecord = new TsFiles();
                newRecord.setNodeId(fileRecord.getNodeId());
                newRecord.setTaskId(fileRecord.getTaskId());
                newRecord.setIsFile(fileRecord.getIsFile());
                newRecord.setParentId(parentId);
                newRecord.setFileName(targetFileName);
                newRecord.setWorkPath(newWorkPath);
                newRecord.setBackupPath(fileRecord.getBackupPath());
                newRecord.setFileSize(fileRecord.getFileSize());
                newRecord.setKeywords(fileRecord.getKeywords());
                newRecord.setDescription(fileRecord.getDescription());
                newRecord.setUploadTime(new Timestamp(System.currentTimeMillis())); // 上传时间
                newRecord.setUploader(fileRecord.getUploader()); // 上传人
                tsFilesMapper.insert(newRecord);
                // 如果是文件夹，递归删除子项
                if ("FOLDER".equals(fileRecord.getIsFile())) {
                    // 先移动子项再删除原记录
                    moveChildrenRecords(oldWorkPath, newWorkPath + targetFileName + "/", newRecord.getId(), type, fileRecord.getId());
                }
            }
            // 删除原记录
            tsFilesMapper.deleteById(fileRecord.getId());

        } else {
            // 重命名模式：插入新记录
            TsFiles newRecord = new TsFiles();
            newRecord.setNodeId(fileRecord.getNodeId());
            newRecord.setTaskId(fileRecord.getTaskId());
            newRecord.setIsFile(fileRecord.getIsFile());
            newRecord.setParentId(parentId);
            newRecord.setFileName(targetFileName);
            newRecord.setWorkPath(newWorkPath);
            newRecord.setBackupPath(fileRecord.getBackupPath());
            newRecord.setFileSize(fileRecord.getFileSize());
            newRecord.setKeywords(fileRecord.getKeywords());
            newRecord.setDescription(fileRecord.getDescription());
            newRecord.setUploadTime(new Timestamp(System.currentTimeMillis())); // 上传时间
            newRecord.setUploader(fileRecord.getUploader()); // 上传人
            tsFilesMapper.insert(newRecord);
            // 删除原记录
            tsFilesMapper.deleteById(fileRecord.getId());
        }
    }

    /**
     * 递归移动子项记录（覆盖模式下先移动后删除）
     */
    private void moveChildrenRecords(String oldPrefix, String newPrefix, String parentId, String type, String oldparentId) {
        //查询原来的
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().eq("parent_id", oldparentId));

        for (TsFiles child : children) {
            // 构建子项新路径
            String childNewPath = newPrefix;

            // 处理子项记录
            if ("0".equals(type)) {
                // 覆盖模式：更新或插入
                TsFiles existingChild = tsFilesMapper.selectOne(
                        new QueryWrapper<TsFiles>().eq("work_path", childNewPath)
                                .eq("file_name", child.getFileName()));

                if (existingChild != null) {
                    existingChild.setParentId(parentId);
                    existingChild.setWorkPath(childNewPath);
                    existingChild.setUpdateTime(new Timestamp(System.currentTimeMillis()));
                    tsFilesMapper.updateById(existingChild);
                } else {
                    TsFiles newChild = new TsFiles();
                    newChild.setNodeId(child.getNodeId());
                    newChild.setTaskId(child.getTaskId());
                    newChild.setIsFile(child.getIsFile());
                    newChild.setParentId(parentId);
                    newChild.setFileName(child.getFileName());
                    newChild.setWorkPath(childNewPath);
                    newChild.setBackupPath(child.getBackupPath());
                    newChild.setFileSize(child.getFileSize());
                    newChild.setKeywords(child.getKeywords());
                    newChild.setDescription(child.getDescription());
                    newChild.setUploadTime(new Timestamp(System.currentTimeMillis()));
                    newChild.setUploader(child.getUploader());
                    tsFilesMapper.insert(newChild);
                    // 递归处理子文件夹
                    if ("FOLDER".equals(child.getIsFile())) {
                        moveChildrenRecords(oldPrefix, newPrefix + newChild.getFileName() + "/", newChild.getId(), type, child.getId());
                    }
                }
            }
        }
    }

    /**
     * 检查路径是否为空
     */
    private boolean isPathEmpty(Path path) throws IOException {
        if (Files.isDirectory(path)) {
            try (DirectoryStream<Path> dirStream = Files.newDirectoryStream(path)) {
                return !dirStream.iterator().hasNext();
            }
        }
        return false;
    }


    /**********************************************************移动文件或者文件夹结束***********************************************************************************/


    /**********************************************************复制文件或者文件夹开始***********************************************************************************/


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String copyFileFolder(MoveCopyFileFolderRequest reques) throws IOException {
        // 1. 获取存储根路径
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(
                new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        Path rootPath = Paths.get(config.getValue());

        String newPath = reques.getNewPath();
        String oldpaths = reques.getOldpaths();
        String parentId = reques.getParentId();
        String rename = reques.getRename();
        String type = reques.getType();
        String[] newFileNames = reques.getNewFileName().split(",");
        // 数组转集合
        List<String> newFileNameList = Arrays.asList(newFileNames);

        // 2. 构建目标父目录的物理路径
        Path targetParentPhysicalPath = buildPhysicalPath(newPath, rootPath);

        // 3. 循环处理所有选中的文件/文件夹
        for (String fileName : newFileNameList) {
            // 构建原文件物理路径
            Path sourcePath = buildPhysicalPath(oldpaths + "/" + fileName, rootPath);


            if (!Files.exists(sourcePath)) {
                throw new IOException("源文件不存在: " + sourcePath);
            }

            // ==== 新增关键校验 ====
            // 1. 禁止移动到自身
            if (sourcePath.equals(targetParentPhysicalPath)) {
                throw new IllegalArgumentException("不能复制到自己所在目录");
            }

            // 2. 禁止文件夹移动到子目录
            if (Files.isDirectory(sourcePath)) {
                if (targetParentPhysicalPath.startsWith(sourcePath)) {
                    throw new IllegalArgumentException("禁止将文件夹复制到自己内部");
                }
            }

            // 生成目标文件名（处理重命名和冲突）
            String targetFileName = generateTargetFileName(targetParentPhysicalPath, fileName, rename, type);
            Path targetPath = targetParentPhysicalPath.resolve(targetFileName);

            // 3. 再次检查自移动
            if (sourcePath.equals(targetPath)) {
                throw new IllegalArgumentException("不能复制到自己当前位置");
            }
            // 执行物理复制
            copyPhysicalFile(sourcePath, targetPath, type);

            // 5. 插入新记录到数据库
            insertDatabaseRecord(parentId, oldpaths, fileName, newPath, targetFileName, type);
        }
        return "复制成功";
    }


    //========================= 核心辅助方法 =========================//

    /**
     * 复制物理文件/文件夹（带递归）
     */
    private void copyPhysicalFile(Path source, Path target, String type) throws IOException {
        if (Files.isDirectory(source)) {
            Files.createDirectories(target);
            try (DirectoryStream<Path> dirStream = Files.newDirectoryStream(source)) {
                for (Path child : dirStream) {
                    String childName = child.getFileName().toString();
                    copyPhysicalFile(child, target.resolve(childName), type);
                }
            }
        } else {
            if ("0".equals(type)) { // 覆盖模式
                Files.copy(source, target, StandardCopyOption.REPLACE_EXISTING);
            } else { // 重命名模式
                if (Files.exists(target)) throw new IOException("目标文件已存在: " + target);
                Files.copy(source, target);
            }
        }
    }

    /**
     * 插入数据库记录（主记录）
     */
    private void insertDatabaseRecord(String parentId, String oldpaths, String fileName,
                                      String newPath, String targetFileName, String type) {
        // 获取用户信息
        // 获取当前用户和时间
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        Timestamp currentTime = Timestamp.valueOf(LocalDateTime.now());
        //判断是不是覆盖  如果是覆盖  更新时间


        // 查询原记录
        TsFiles original = tsFilesMapper.selectOne(new QueryWrapper<TsFiles>()
                .eq("work_path", oldpaths)
                .eq("file_name", fileName)
        );
        //查询新纪录
        TsFiles neworiginal = tsFilesMapper.selectOne(new QueryWrapper<TsFiles>()
                .eq("work_path", newPath)
                .eq("file_name", targetFileName)
        );

        //如果是覆盖 并且通过新路径和名称可以查询到  那就更新时间
        if ("0".equals(type) && neworiginal != null) {

            neworiginal.setUpdateTime(currentTime);
            tsFilesMapper.updateById(neworiginal);
            if ("FOLDER".equals(original.getIsFile())) {
                copyChildrenRecords(
                        formatDbPath(oldpaths),
                        formatDbPath(newPath),
                        original.getId(),
                        neworiginal.getId(),
                        type
                );
            }

        } else {
            // 创建新记录
            TsFiles newRecord = new TsFiles();
            newRecord.setNodeId(original.getNodeId());
            newRecord.setTaskId(original.getTaskId());
            newRecord.setIsFile(original.getIsFile());
            newRecord.setParentId(parentId);
            newRecord.setFileName(targetFileName);
            newRecord.setWorkPath(formatDbPath(newPath));
            newRecord.setFileSize(original.getFileSize());
            newRecord.setKeywords(original.getKeywords());
            newRecord.setDescription(original.getDescription());
            newRecord.setUploadTime(currentTime);
            newRecord.setUploader(loginuser.getUsername());

            tsFilesMapper.insert(newRecord);
            // 递归处理子项
            if ("FOLDER".equals(original.getIsFile())) {
                copyChildrenRecords(
                        formatDbPath(oldpaths),
                        formatDbPath(newPath + newRecord.getFileName() + "/"),
                        original.getId(),
                        newRecord.getId(),
                        type
                );
            }
        }


    }

    /**
     * 递归复制子项数据库记录（关键优化点）
     */
    private void copyChildrenRecords(String oldPrefix, String newPrefix,
                                     String originalParentId, String newParentId, String type) {
        List<TsFiles> children = tsFilesMapper.selectList(
                new QueryWrapper<TsFiles>().eq("parent_id", originalParentId)
        );
        // 获取当前用户和时间
        UsernamePasswordAuthenticationToken authentication =
                (UsernamePasswordAuthenticationToken) SecurityContextHolder.getContext().getAuthentication();
        LoginUser loginuser = (LoginUser) authentication.getPrincipal();
        Timestamp currentTime = Timestamp.valueOf(LocalDateTime.now());

        for (TsFiles child : children) {
            // 处理当前子项
            TsFiles processedChild = processChildRecord(
                    child, newParentId, newPrefix, oldPrefix, type, loginuser, currentTime
            );

            // 递归处理子文件夹
            if ("FOLDER".equals(child.getIsFile())) {
                copyChildrenRecords(
                        oldPrefix,
                        newPrefix,
                        child.getId(),
                        processedChild.getId(), // 使用新记录的ID作为父ID
                        type
                );
            }
        }
    }

    /**
     * 处理单个子项记录
     */
    private TsFiles processChildRecord(TsFiles child, String newParentId, String newPrefix,
                                       String oldPrefix, String type, LoginUser user, Timestamp now) {
        if ("0".equals(type)) { // 覆盖模式
            TsFiles existing = tsFilesMapper.selectOne(
                    new QueryWrapper<TsFiles>()
                            .eq("work_path", buildNewWorkPath(child, oldPrefix, newPrefix))
                            .eq("file_name", child.getFileName())
            );
            if (existing != null) {
                updateExisting(existing, child, now, user);
                return existing;
            } else {
                return insertNewChild(child, newParentId, buildNewWorkPath(child, oldPrefix, newPrefix),
                        user, now, null);
            }
        } else { // 重命名模式
            return insertNewChild(child, newParentId, newPrefix,
                    user, now, child.getFileName());
        }
    }

    /**
     * 更新现有记录（覆盖模式）
     */
    private void updateExisting(TsFiles existing, TsFiles source, Timestamp time, LoginUser user) {
        existing.setFileSize(source.getFileSize());
        existing.setDescription(source.getDescription());
        existing.setUpdateTime(time);
        existing.setUploader(user.getUsername());
        tsFilesMapper.updateById(existing);
    }

    /**
     * 插入新子项记录
     */
    private TsFiles insertNewChild(TsFiles source, String parentId, String newWorkPath,
                                   LoginUser user, Timestamp time, String customName) {
        TsFiles newChild = new TsFiles();
        newChild.setNodeId(source.getNodeId());
        newChild.setTaskId(source.getTaskId());
        newChild.setIsFile(source.getIsFile());
        newChild.setParentId(parentId);
        newChild.setFileName(customName != null ? customName : source.getFileName());
        newChild.setWorkPath(newWorkPath);
        newChild.setFileSize(source.getFileSize());
        newChild.setKeywords(source.getKeywords());
        newChild.setDescription(source.getDescription());
        newChild.setUploadTime(time);
        newChild.setUploader(user.getUsername());
        tsFilesMapper.insert(newChild);
        return newChild;
    }

//========================= 工具方法 =========================//

    /**
     * 构建安全的新路径（精确替换前缀）
     */
    private String buildNewWorkPath(TsFiles child, String oldPrefix, String newPrefix) {
        // 确保旧前缀是父路径（如 /a/b/ 而不是 /a/b/file.txt）
        String childPath = child.getWorkPath();

        // 关键修复：严格匹配父路径前缀
        if (!childPath.startsWith(oldPrefix)) {
            throw new IllegalStateException("路径前缀不匹配: " + childPath);
        }

        // 计算子项相对路径（剥离旧前缀）
        String relativePath = childPath.substring(oldPrefix.length());

        // 拼接新路径（避免重复层级）
        return newPrefix + relativePath;
    }

    /**
     * 格式化数据库路径（统一以/结尾）
     */
    private String formatDbPath(String path) {
        if (path == null || path.isEmpty()) return "/";
        return path.endsWith("/") ? path : path + "/";
    }

    /**
     * 获取当前用户（封装重复逻辑）
     */
    private LoginUser getCurrentUser() {
        return (LoginUser) SecurityContextHolder.getContext().getAuthentication().getPrincipal();
    }

    /**
     * 关键安全校验
     */
    private void validateCopySafety(Path source, Path targetParent) {
        // 禁止复制到自身目录
        if (source.equals(targetParent)) {
            throw new IllegalArgumentException("不能复制到自身目录");
        }

        // 禁止文件夹嵌套复制
        if (Files.isDirectory(source) && targetParent.startsWith(source)) {
            throw new IllegalArgumentException("禁止将文件夹复制到自身内部");
        }
    }


    /**********************************************************复制文件或者文件夹结束***********************************************************************************/

    /**********************************
     * 用途说明: 查询本地和备份空间结构树
     * 参数说明  taskId 节点ID
     * 参数说明  nodeId 任务ID
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回双树数据
     ***********************************/
    @Override
    public DualTreeResponse listLocalAndBackup(String taskId, String nodeId) {
        // 记录方法入参
        LOGGER.info("Starting to build dual trees for taskId={}, nodeId={}", taskId, nodeId);
        // 1. 批量查询所有相关节点
        QueryWrapper<TsFiles> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("task_id", taskId)
                .eq("node_id", nodeId);
        List<TsFiles> allNodes = tsFilesMapper.selectList(queryWrapper);

        // 2. 构建内存索引提升查询效率
        Map<String, List<TsFiles>> parentChildrenMap = allNodes.stream()
                .collect(Collectors.groupingBy(TsFiles::getParentId));
        LOGGER.debug("使用｛｝个条目构建父子映射", parentChildrenMap.size());

        // 3. 双树构建容器
        List<TreeDTO> localTrees = new ArrayList<>();
        List<TreeDTO> minioTrees = new ArrayList<>();

        // 4. 从顶级节点（parentId为00）开始构建树
        List<TsFiles> rootNodes = parentChildrenMap.get("00");
        if (rootNodes == null || rootNodes.isEmpty()) {
            LOGGER.warn("找不到的根节点 taskId={}, nodeId={}", taskId, nodeId);
            return new DualTreeResponse(localTrees, minioTrees);
        }

        if (rootNodes != null) {
            for (TsFiles rootNode : rootNodes) {
                // 构建本地树
                TreeDTO localTree = buildTree(rootNode, parentChildrenMap, true);
                if (localTree != null) {
                    localTrees.add(localTree);
                    LOGGER.debug("添加了本地树节点: {}", localTree.getId());
                }

                // 构建Minio树
                TreeDTO minioTree = buildTree(rootNode, parentChildrenMap, false);
                if (minioTree != null) {
                    minioTrees.add(minioTree);
                    LOGGER.debug("添加了MINIO树节点: {}", minioTree.getId());
                }
            }
        }
        LOGGER.info("Tree construction completed. Local nodes: {}, MinIO nodes: {}", localTrees.size(), minioTrees.size());
        return new DualTreeResponse(localTrees, minioTrees);
    }

    /**
     * 递归构建树形结构（内存操作，不再查询数据库）
     *
     * @param current           当前节点
     * @param parentChildrenMap 父子关系映射表
     * @param isLocal           是否是本地树
     * @return 有效的树节点（路径不为空）
     */
    private TreeDTO buildTree(TsFiles current, Map<String, List<TsFiles>> parentChildrenMap, boolean isLocal) {
        // 1. 转换为DTO并检查路径有效性
        TreeDTO dto = convertToDTO(current, isLocal);
        if (dto.getPath() == null || dto.getPath().trim().isEmpty()) {
            LOGGER.warn("由于路径为空，跳过节点｛｝", current.getId());
            return null;
        }

        // 2. 查找子节点（从内存映射中获取）
        List<TsFiles> children = parentChildrenMap.get(current.getId());
        if (children != null) {
            for (TsFiles child : children) {
                TreeDTO childDto = buildTree(child, parentChildrenMap, isLocal);
                if (childDto != null) {
                    dto.getChildren().add(childDto);
                }
            }
        }

        return dto;
    }

    /**
     * 将 FileNode 转换为 TreeDTO
     *
     * @param node    当前节点
     * @param isLocal 是否是本地树（true: workPath, false: backupPath）
     * @return 转换后的 DTO
     */
    private TreeDTO convertToDTO(TsFiles node, boolean isLocal) {
        //查询字典表获取压缩文件后缀
        QueryWrapper<SysDictionaryItems> queryWrapperSysDictionary = new QueryWrapper<>();
        queryWrapperSysDictionary.eq("parentcode", "compressType");
        queryWrapperSysDictionary.orderByAsc("orderno");
        List<SysDictionaryItems> sysDictionaryItems = sysDictionaryItemsMapper.selectList(queryWrapperSysDictionary);


        TreeDTO dto = new TreeDTO();
        // 复制公共字段
        dto.setId(node.getId());
        dto.setNodeId(node.getNodeId());
        dto.setTaskId(node.getTaskId());
        dto.setIsFile(node.getIsFile());
        dto.setParentId(node.getParentId());
        dto.setFileName(node.getFileName());
        dto.setFileSize(node.getFileSize());
        dto.setKeywords(node.getKeywords());
        dto.setDescription(node.getDescription());
        dto.setUploadTime(node.getUploadTime());
        dto.setUploader(node.getUploader());
        dto.setUpdateTime(node.getUpdateTime());

        //查询本地
        if (isLocal) {
            String workPath = node.getWorkPath();
            if (StringUtils.isEmpty(workPath)) {
                dto.setPath(null);
                return dto;
            }
            String fileNameData = node.getFileName();
            //主要是用于文件路径加名称
            String path = workPath + fileNameData;
            //准备获取文件的信息
            if ("FILE".equals(node.getIsFile())) {
                LOGGER.info("查询本地树的时候" + path);
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("local");
                FileItemResult fileItemResult = fileService.getFileItem(path);
                if (fileItemResult != null || fileItemResult.getName() != null) {
                    dto.setUrl(fileItemResult.getUrl());
                    //如果是压缩文件 类型就给zip
                    boolean isValid = hasValidExtension(fileItemResult.getName(), sysDictionaryItems);
                    if (isValid) {
                        dto.setType("ZIP");
                    } else {
                        dto.setType(fileItemResult.getType().getValue());
                    }
                }

            }
        } else {
            String backupPath = node.getBackupPath();
            if (backupPath == null || StringUtils.isEmpty(backupPath)) {
                dto.setPath(null);
                return dto;
            }
            String fileNameData = node.getFileName();
            //主要是用于文件路径加名称
            String path = backupPath + fileNameData;
            //准备获取文件的信息
            if ("FILE".equals(node.getIsFile())) {
                LOGGER.info("查询minio树的时候" + path);
                AbstractBaseFileService<?> fileService = storageSourceContext.getByStorageKey("minio");
                FileItemResult fileItemResult = fileService.getFileItem(path);
                if (fileItemResult == null) {
                    dto.setUrl(null);
                    dto.setType(null);

                } else {
                    dto.setUrl(fileItemResult.getUrl());
                    //如果是压缩文件 类型就给zip
                    boolean isValid = hasValidExtension(fileItemResult.getName(), sysDictionaryItems);
                    if (isValid) {
                        dto.setType("ZIP");
                    } else {
                        dto.setType(fileItemResult.getType().getValue());
                    }
                }
            }
        }
        // 设置路径字段
        dto.setPath(isLocal ? node.getWorkPath() : node.getBackupPath());
        return dto;
    }


    /*******************************************读取文件内容经纬度**************************************************************/

    @Override
    public void batchSendNaviOutDataJob(String id, int samTimes, String token) {
        if (currentTaskFuture != null && !currentTaskFuture.isDone()) {
            currentTaskFuture.cancel(true); // 中断之前的任务
        }
        currentTaskFuture = executorService.submit(() -> {
            TsFiles tsFiles = tsFilesMapper.selectById(id);
            StorageSourceConfig config = storageSourceConfigMapper.selectOne(
                    new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
            Path filePath = Paths.get(config.getValue() + tsFiles.getWorkPath() + tsFiles.getFileName());

            try (BufferedReader reader = new BufferedReader(new FileReader(filePath.toFile()))) {
                reader.readLine(); // 跳过标题行

                // 不管 samTimes 如何，先读取第一行数据
                String firstLine = reader.readLine();
                if (firstLine != null) {
                    firstLine = firstLine.trim();
                    String[] values = firstLine.split("\t");
                    if (values.length >= 40) {
                        // 发送第一行数据
                        sendData(token, values, 1);
                    } else {
                        System.err.println("忽略不完整行: " + firstLine);
                    }
                }

                int step = samTimes * 200; // 计算行号间隔
                int lineCount = 1; // 从第一行开始

                while (!Thread.currentThread().isInterrupted()) {
                    lineCount += step; // 直接跳到下一个需要读取的行
                    String line = null;

                    // 读取指定行
                    for (int i = 0; i < step; i++) {
                        line = reader.readLine();
                        if (line == null) break; // 文件结束
                    }

                    if (line == null) break; // 文件结束

                    line = line.trim();
                    String[] values = line.split("\t");
                    if (values.length < 40) {
                        LOGGER.info("忽略不完整行: " + line);
                        continue;
                    }

                    // 发送数据
                    sendData(token, values, lineCount);

                    // 固定休眠 3 秒
                    try {
                        Thread.sleep(3000);
                    } catch (InterruptedException e) {
                        // 如果休眠期间被中断，则退出循环
                        Thread.currentThread().interrupt();
                        break;
                    }
                }

                // 发送完成信号（如果需要）
                if (!Thread.currentThread().isInterrupted()) {
                    //  ServerSendEventServer.sendMessage(token, "COMPLETED");
                }

            } catch (Exception e) {
                // 捕获异常并记录日志
                LOGGER.error("任务执行失败: " + e.getMessage());
                e.printStackTrace();
            }
        });
    }


    // 封装发送数据的逻辑
    private void sendData(String token, String[] values, int lineCount) {
        if (Thread.currentThread().isInterrupted()) {
            return; // 如果线程已被中断，则直接返回
        }

        try {
            SimpleNaviData data = parseLine(values);
            String jsonData = JSONUtil.toJsonStr(data); // 假设 JSONUtil.toJsonStr 不会抛出 IOException
            ServerSendEventServer.sendMessageById(token, jsonData); // 假设 sendMessage 不会抛出 IOException
            LOGGER.info("Line " + lineCount + " sent at: " + System.currentTimeMillis());
        } catch (Exception e) {
            // 捕获所有可能的异常
            LOGGER.error("发送数据失败: " + e.getMessage());
            if (e.getCause() instanceof InterruptedException) {
                Thread.currentThread().interrupt(); // 重新设置中断状态
            }
        }
    }

    // 辅助方法：解析行数据
    private SimpleNaviData parseLine(String[] values) {
        SimpleNaviData data = new SimpleNaviData();
        data.setUtcTime(getValueSafely(values, 8, "0.0")); // UTC_TIME（索引8）
        data.setLat(getValueSafely(values, 17, "0.0"));    // LAT（索引17）
        data.setLon(getValueSafely(values, 18, "0.0"));    // LON（索引18）
        data.setAlt(getValueSafely(values, 19, "0.0"));    // ALT（索引19）
        return data;
    }

    // 安全获取数组值的方法
    private String getValueSafely(String[] values, int index, String defaultValue) {
        return (index < values.length) ? values[index] : defaultValue;
    }


    /**
     * 查询文件内容接口
     *
     * @param id 文件的id
     * @return 文件内容的纯文本（UTF-8 编码）
     */
    @Override
    public String readFileContent(String id) throws IOException {
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        TsFiles tsFiles = tsFilesMapper.selectById(id);
        // 1. 路径标准化与安全校验
        Path targetPath = validateAndNormalizePath(config.getValue() + tsFiles.getWorkPath() + tsFiles.getFileName());

        StringBuilder content = new StringBuilder();

        // 使用缓冲流读取大文件（减少内存占用）
        try (BufferedReader reader = Files.newBufferedReader(targetPath, StandardCharsets.UTF_8)) {
            String line;
            while ((line = reader.readLine()) != null) {
                content.append(line).append("\n");
            }
        }
        return content.toString().trim(); // 移除末尾多余换行
    }


    /**
     * 验证路径是否合法并转换为标准化路径
     */
    private Path validateAndNormalizePath(String filePath) throws SecurityException {
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));

        Path targetPath = Paths.get(filePath).normalize();
        Path baseDirPath = Paths.get(config.getValue());

        if (!targetPath.startsWith(baseDirPath)) {
            throw new SecurityException("路径越界: " + targetPath);
        }
        // 防止路径遍历攻击（如 workPath 包含 ".."）
        if (targetPath.toString().contains("..")) {
            throw new SecurityException("非法路径: " + targetPath);
        }

        return targetPath;
    }


    /**
     * 保存文件内容
     *
     * @param id      文件的id
     * @param content 新的文件内容（HTML/文本）
     * @throws SecurityException 路径非法时抛出
     * @throws IOException       文件写入失败时抛出
     */
    public void saveFileContent(String id, String content) throws SecurityException, IOException {
        StorageSourceConfig config = storageSourceConfigMapper.selectOne(new QueryWrapper<StorageSourceConfig>().eq("name", "filePath"));
        TsFiles tsFiles = tsFilesMapper.selectById(id);
        // 1. 路径标准化与安全校验
        Path targetPath = validateAndNormalizePath(config.getValue() + tsFiles.getWorkPath() + tsFiles.getFileName());
        // 2. 确保父目录存在
        Path parentDir = targetPath.getParent();
        if (parentDir != null && !Files.exists(parentDir)) {
            Files.createDirectories(parentDir);
        }
        // 3. 写入文件内容（UTF-8 编码）
        // 使用缓冲流写入大文件（支持流式上传）
        try (BufferedWriter writer = Files.newBufferedWriter(targetPath, StandardCharsets.UTF_8)) {
            writer.write(content);
        }
    }

}



